------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                           E B N F . N O D E S                            --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Namet;
with Output;

package body EBNF.Nodes
   --# own State is Table, Last_Id;
is

   Max_Node_Id : constant := Types.Max_Node_Id;

   subtype Node_Range is Types.Node_Id range Types.No_Node + 1 .. Max_Node_Id;

   type Table_Type is array (Node_Range) of Node_Entry;
   Table   : Table_Type    := Table_Type'(Node_Range => Default_Node);
   Last_Id : Types.Node_Id := Table'First;

   procedure Increment_Last
   --# global in out Last_Id;
   --# post Last_Id > 0;
   is
   begin
      if Last_Id < Types.Node_Id'Last then
         Last_Id := Last_Id + 1;
      else
         null; --  XXX
      end if;
   end Increment_Last;

   function Kind (N : Types.Node_Id) return Node_Kind
   --# global in Table;
   --# pre N > Types.No_Node;
   is
   begin
      return Table (N).Kind;
   end Kind;

   procedure Set_Kind (N : Types.Node_Id; V : Node_Kind)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      Table (N).Kind := V;
   end Set_Kind;

   function Loc (N : Types.Node_Id) return Locations.Location
   --# global in Table;
   --# pre N > Types.No_Node;
   is
   begin
      return Table (N).Loc;
   end Loc;

   procedure Set_Loc (N : Types.Node_Id; V : Locations.Location)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      Table (N).Loc := V;
   end Set_Loc;

   function Next_Node (N : Types.Node_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   is
      Result : Types.Node_Id;
   begin
      pragma Assert
        (False
        or else Table (N).Kind = K_Node_Id
        or else Table (N).Kind = K_Identifier
        or else Table (N).Kind = K_Meta_Identifier
        or else Table (N).Kind = K_Terminal_String
        or else Table (N).Kind = K_Definitions_List
        or else Table (N).Kind = K_Single_Definition
        or else Table (N).Kind = K_Syntactic_Term
        or else Table (N).Kind = K_Syntactic_Exception
        or else Table (N).Kind = K_Syntactic_Factor
        or else Table (N).Kind = K_Syntactic_Primary
        or else Table (N).Kind = K_Optional_Sequence
        or else Table (N).Kind = K_Repeated_Sequence
        or else Table (N).Kind = K_Grouped_Sequence
        or else Table (N).Kind = K_Syntax_Rule
        or else Table (N).Kind = K_Syntax
        or else Table (N).Kind = K_Empty_Sequence);

      Result := Table (N).L (1);
      return Result;
   end Next_Node;

   procedure Set_Next_Node (N : Types.Node_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert
        (False
        or else Table (N).Kind = K_Node_Id
        or else Table (N).Kind = K_Identifier
        or else Table (N).Kind = K_Meta_Identifier
        or else Table (N).Kind = K_Terminal_String
        or else Table (N).Kind = K_Definitions_List
        or else Table (N).Kind = K_Single_Definition
        or else Table (N).Kind = K_Syntactic_Term
        or else Table (N).Kind = K_Syntactic_Exception
        or else Table (N).Kind = K_Syntactic_Factor
        or else Table (N).Kind = K_Syntactic_Primary
        or else Table (N).Kind = K_Optional_Sequence
        or else Table (N).Kind = K_Repeated_Sequence
        or else Table (N).Kind = K_Grouped_Sequence
        or else Table (N).Kind = K_Syntax_Rule
        or else Table (N).Kind = K_Syntax
        or else Table (N).Kind = K_Empty_Sequence);

      Table (N).L (1) := V;
   end Set_Next_Node;

   function First_Node (N : Types.List_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_List_Id);

      return Table (N).L (1);
   end First_Node;

   procedure Set_First_Node (N : Types.List_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_List_Id);

      Table (N).L (1) := V;
   end Set_First_Node;

   function Last_Node (N : Types.List_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_List_Id);

      return Table (N).L (2);
   end Last_Node;

   procedure Set_Last_Node (N : Types.List_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_List_Id);

      Table (N).L (2) := V;
   end Set_Last_Node;

   function Name (N : Types.Node_Id) return Types.Name_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Name;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Identifier);

      return Table (N).N;
   end Name;

   procedure Set_Name (N : Types.Node_Id; V : Types.Name_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Identifier);

      Table (N).N := V;
   end Set_Name;

   function Identifier (N : Types.Node_Id) return Types.List_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Meta_Identifier);

      return Table (N).L (2);
   end Identifier;

   procedure Set_Identifier (N : Types.Node_Id; V : Types.List_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Meta_Identifier);

      Table (N).L (2) := V;
   end Set_Identifier;

   function The_String (N : Types.Node_Id) return Types.Name_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Name;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Terminal_String);

      return Table (N).N;
   end The_String;

   procedure Set_The_String (N : Types.Node_Id; V : Types.Name_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Terminal_String);

      Table (N).N := V;
   end Set_The_String;

   function Single_Definition (N : Types.Node_Id) return Types.List_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Definitions_List);

      return Table (N).L (2);
   end Single_Definition;

   procedure Set_Single_Definition (N : Types.Node_Id; V : Types.List_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Definitions_List);

      Table (N).L (2) := V;
   end Set_Single_Definition;

   function Syntactic_Term (N : Types.Node_Id) return Types.List_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Single_Definition);

      return Table (N).L (2);
   end Syntactic_Term;

   procedure Set_Syntactic_Term (N : Types.Node_Id; V : Types.List_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Single_Definition);

      Table (N).L (2) := V;
   end Set_Syntactic_Term;

   function Syntactic_Factor (N : Types.Node_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert
        (False
        or else Table (N).Kind = K_Syntactic_Term
        or else Table (N).Kind = K_Syntactic_Exception);

      return Table (N).L (2);
   end Syntactic_Factor;

   procedure Set_Syntactic_Factor (N : Types.Node_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert
        (False
        or else Table (N).Kind = K_Syntactic_Term
        or else Table (N).Kind = K_Syntactic_Exception);

      Table (N).L (2) := V;
   end Set_Syntactic_Factor;

   function Syntactic_Exception (N : Types.Node_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntactic_Term);

      return Table (N).L (3);
   end Syntactic_Exception;

   procedure Set_Syntactic_Exception (N : Types.Node_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntactic_Term);

      Table (N).L (3) := V;
   end Set_Syntactic_Exception;

   function Syntactic_Primary (N : Types.Node_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntactic_Factor);

      return Table (N).L (2);
   end Syntactic_Primary;

   procedure Set_Syntactic_Primary (N : Types.Node_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntactic_Factor);

      Table (N).L (2) := V;
   end Set_Syntactic_Primary;

   function A_Element (N : Types.Node_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntactic_Primary);

      return Table (N).L (2);
   end A_Element;

   procedure Set_Element (N : Types.Node_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntactic_Primary);

      Table (N).L (2) := V;
   end Set_Element;

   function Definitions_List (N : Types.Node_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert
        (False
        or else Table (N).Kind = K_Optional_Sequence
        or else Table (N).Kind = K_Repeated_Sequence
        or else Table (N).Kind = K_Grouped_Sequence
        or else Table (N).Kind = K_Syntax_Rule);

      return Table (N).L (2);
   end Definitions_List;

   procedure Set_Definitions_List (N : Types.Node_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert
        (False
        or else Table (N).Kind = K_Optional_Sequence
        or else Table (N).Kind = K_Repeated_Sequence
        or else Table (N).Kind = K_Grouped_Sequence
        or else Table (N).Kind = K_Syntax_Rule);

      Table (N).L (2) := V;
   end Set_Definitions_List;

   function Meta_Identifier (N : Types.Node_Id) return Types.Node_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntax_Rule);

      return Table (N).L (3);
   end Meta_Identifier;

   procedure Set_Meta_Identifier (N : Types.Node_Id; V : Types.Node_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntax_Rule);

      Table (N).L (3) := V;
   end Set_Meta_Identifier;

   function Syntax_Rules (N : Types.Node_Id) return Types.List_Id
   --# global in Table;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntax);

      return Table (N).L (2);
   end Syntax_Rules;

   procedure Set_Syntax_Rules (N : Types.Node_Id; V : Types.List_Id)
   --# global in out Table;
   --# pre N > Types.No_Node;
   is
   begin
      pragma Assert (False or else Table (N).Kind = K_Syntax);

      Table (N).L (2) := V;
   end Set_Syntax_Rules;

   ---------------
   -- W_Node_Id --
   ---------------

   procedure W_Node_Id (N : Types.Node_Id) is
   --# hide W_Node_Id;
   begin
      if N /= Types.No_Node then
         W_Node (N);
      end if;
   end W_Node_Id;

   ---------------
   -- W_Indents --
   ---------------

   procedure W_Indents is
   begin
      for J in Integer range 1 .. N_Indents loop
         Output.Write_Str (" ");
      end loop;
   end W_Indents;

   ---------------
   -- W_List_Id --
   ---------------

   procedure W_List_Id (L : Types.List_Id)
   --# global in     Table;
   --#        in out N_Indents;
   --#        in out Output.State;
   is
      E : Types.Node_Id;
   begin
      if L /= Types.No_List then
         E := First_Node (L);
         while E /= Types.No_Node loop
            W_Node_Id (E);
            E := Next_Node (E);
         end loop;
      end if;
   end W_List_Id;

   ----------------------
   -- W_Node_Attribute --
   ----------------------

   procedure W_Node_Attribute
     (A : String;
      K : String;
      V : Spark.Strings.SparkString;
      N : Types.Int)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   is
      --# hide W_Node_Attribute;
   begin
      --  XXX Why skipping these cases ?
      if A /= "Next_Node" then
         Output.Increment_Indentation;
         W_Indents;
         Output.Write_Str (A);
         Output.Write_Char (' ');
         Output.Write_Str (K);
         Output.Write_Char (' ');
         if K = "Name_Id" then
            Output.Write_Str ("""");
            Output.Write_SparkStr (V);
            Output.Write_Str ("""");
         else
            Output.Write_SparkStr (V);
            Output.Write_Eol;
         end if;

         if K = "Node_Id" then
            W_Node_Id (N);
         elsif K = "List_Id" then
            W_List_Id (N);
         end if;

         Output.Decrement_Indentation;
      end if;
   end W_Node_Attribute;

   ---------------------
   -- Node_Kind_Image --
   ---------------------

   function Node_Kind_Image
     (N    : Node_Kind)
      return Spark.Strings.SparkString
   is
   --# hide Node_Kind_Image;

      S      : String := Node_Kind'Image (N);
      Result : Spark.Strings.SparkString;
   begin
      for J in Integer range S'Range loop
         if S (J) = '_' then
            S (J) := ' ';
         end if;
      end loop;

      Spark.Strings.CopyString (Result, S);
      Result := Spark.Strings.Substring (Result, 3, Result.Length);
      return Result;
   end Node_Kind_Image;

   -------------------
   -- W_Node_Header --
   -------------------

   procedure W_Node_Header (N : Types.Node_Id)
   --# global in     N_Indents;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   is
      Loc_Img : Spark.Strings.SparkString;

   begin
      W_Indents;
      Output.Write_Int (N);
      Output.Write_Char (' ');
      Output.Write_SparkStr (Node_Kind_Image (Kind (N)));
      Output.Write_Char (' ');
      Locations.Location_Image (Loc (N), Loc_Img);
      Output.Write_Line (Loc_Img.Text);
   end W_Node_Header;

   procedure W_Identifier (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute ("Name", "Name_Id", Namet.Name_String (Name (N)), N);
   end W_Identifier;

   procedure W_Meta_Identifier (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Identifier",
         "List_Id",
         Types.Int_Image (Identifier (N)),
         Identifier (N));
   end W_Meta_Identifier;

   procedure W_Terminal_String (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("The_String",
         "Name_Id",
         Namet.Name_String (The_String (N)),
         N);
   end W_Terminal_String;

   procedure W_Definitions_List (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Single_Definition",
         "List_Id",
         Types.Int_Image (Single_Definition (N)),
         Single_Definition (N));
   end W_Definitions_List;

   procedure W_Single_Definition (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Syntactic_Term",
         "List_Id",
         Types.Int_Image (Syntactic_Term (N)),
         Syntactic_Term (N));
   end W_Single_Definition;

   procedure W_Syntactic_Term (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Syntactic_Factor",
         "Node_Id",
         Types.Int_Image (Syntactic_Factor (N)),
         Syntactic_Factor (N));
      W_Node_Attribute
        ("Syntactic_Exception",
         "Node_Id",
         Types.Int_Image (Syntactic_Exception (N)),
         Syntactic_Exception (N));
   end W_Syntactic_Term;

   procedure W_Syntactic_Exception (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Syntactic_Factor",
         "Node_Id",
         Types.Int_Image (Syntactic_Factor (N)),
         Syntactic_Factor (N));
   end W_Syntactic_Exception;

   procedure W_Syntactic_Factor (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Syntactic_Primary",
         "Node_Id",
         Types.Int_Image (Syntactic_Primary (N)),
         Syntactic_Primary (N));
   end W_Syntactic_Factor;

   procedure W_Syntactic_Primary (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Element",
         "Node_Id",
         Types.Int_Image (A_Element (N)),
         A_Element (N));
   end W_Syntactic_Primary;

   procedure W_Optional_Sequence (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Definitions_List",
         "Node_Id",
         Types.Int_Image (Definitions_List (N)),
         Definitions_List (N));
   end W_Optional_Sequence;

   procedure W_Repeated_Sequence (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Definitions_List",
         "Node_Id",
         Types.Int_Image (Definitions_List (N)),
         Definitions_List (N));
   end W_Repeated_Sequence;

   procedure W_Grouped_Sequence (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Definitions_List",
         "Node_Id",
         Types.Int_Image (Definitions_List (N)),
         Definitions_List (N));
   end W_Grouped_Sequence;

   procedure W_Syntax_Rule (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Definitions_List",
         "Node_Id",
         Types.Int_Image (Definitions_List (N)),
         Definitions_List (N));
      W_Node_Attribute
        ("Meta_Identifier",
         "Node_Id",
         Types.Int_Image (Meta_Identifier (N)),
         Meta_Identifier (N));
   end W_Syntax_Rule;

   procedure W_Syntax (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
      W_Node_Attribute
        ("Syntax_Rules",
         "List_Id",
         Types.Int_Image (Syntax_Rules (N)),
         Syntax_Rules (N));
   end W_Syntax;

   procedure W_Empty_Sequence (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      W_Node_Header (N);
      W_Node_Attribute
        ("Next_Node",
         "Node_Id",
         Types.Int_Image (Next_Node (N)),
         Next_Node (N));
   end W_Empty_Sequence;

   procedure W_Node (N : Types.Node_Id)
   --# global in     Output.Space_Increment;
   --#        in     Table;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;
   is
   begin
      case Kind (N) is
      when K_Identifier =>
         W_Identifier (N);
      when K_Meta_Identifier =>
         W_Meta_Identifier (N);
      when K_Terminal_String =>
         W_Terminal_String (N);
      when K_Definitions_List =>
         W_Definitions_List (N);
      when K_Single_Definition =>
         W_Single_Definition (N);
      when K_Syntactic_Term =>
         W_Syntactic_Term (N);
      when K_Syntactic_Exception =>
         W_Syntactic_Exception (N);
      when K_Syntactic_Factor =>
         W_Syntactic_Factor (N);
      when K_Syntactic_Primary =>
         W_Syntactic_Primary (N);
      when K_Optional_Sequence =>
         W_Optional_Sequence (N);
      when K_Repeated_Sequence =>
         W_Repeated_Sequence (N);
      when K_Grouped_Sequence =>
         W_Grouped_Sequence (N);
      when K_Syntax_Rule =>
         W_Syntax_Rule (N);
      when K_Syntax =>
         W_Syntax (N);
      when K_Empty_Sequence =>
         W_Empty_Sequence (N);
      when others =>
         null;
      end case;
   end W_Node;

   -------------------------
   -- Append_Node_To_List --
   -------------------------

   procedure Append_Node_To_List (E : Types.Node_Id; L : Types.List_Id)
   --# global in out Table;
   --# pre L > Types.No_Node;
   is
      Last_N : Types.Node_Id;
   begin
      Last_N := Last_Node (L);
      if Types.No (Last_N) then
         Set_First_Node (L, E);
      else
         Set_Next_Node (Last_N, E);
      end if;
      Last_N := E;
      while Types.Present (Last_N) loop
         Set_Last_Node (L, Last_N);
         Last_N := Next_Node (Last_N);
      end loop;
   end Append_Node_To_List;

   --------------
   -- Is_Empty --
   --------------

   function Is_Empty (L : Types.List_Id) return Boolean
   --# global in Table;
   is
   begin
      return L = Types.No_List or else Types.No (First_Node (L));
   end Is_Empty;

   --------------
   -- New_Node --
   --------------

   procedure New_Node
     (The_Kind : Node_Kind;
      The_Loc  : Locations.Location;
      Result   : out Types.Node_Id)
   --# global in out Last_Id;
   --#        in out Table;
   --# post Result > Types.No_Node;
   is
   begin
      Increment_Last;
      Result         := Last_Id;
      Table (Result) := Default_Node;
      Set_Kind (Result, The_Kind);
      Set_Loc (Result, The_Loc);
   end New_Node;

   --------------
   -- New_List --
   --------------

   procedure New_List
     (The_Kind : Node_Kind;
      The_Loc  : Locations.Location;
      Result   : out Types.List_Id)
   --# global in out Last_Id;
   --#        in out Table;
   --# post Result > Types.No_Node;
   is
   begin
      New_Node (The_Kind, The_Loc, Result);
   end New_List;

end EBNF.Nodes;
