------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                           E B N F . N O D E S                            --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Locations;
with Types;         use type Types.Int;
with Spark.Strings;
--# inherit Locations,
--#         Namet,
--#         Output,
--#         Spark.Strings,
--#         Types;

package EBNF.Nodes
   --# own N_Indents,
   --#     State;
   --# initializes N_Indents,
   --#             State;
is

   type Node_Kind is (
      K_Node_Id,
      K_List_Id,
      K_Identifier,
      K_Meta_Identifier,
      K_Terminal_String,
      K_Definitions_List,
      K_Single_Definition,
      K_Syntactic_Term,
      K_Syntactic_Exception,
      K_Syntactic_Factor,
      K_Syntactic_Primary,
      K_Optional_Sequence,
      K_Repeated_Sequence,
      K_Grouped_Sequence,
      K_Syntax_Rule,
      K_Syntax,
      K_Empty_Sequence);

   --
   --  Node_Id
   --
   --    Next_Node                : Node_Id
   --

   --
   --  List_Id
   --
   --    First_Node               : Node_Id
   --    Last_Node                : Node_Id
   --

   --
   --  Identifier
   --
   --    Next_Node                : Node_Id
   --    Name                     : Name_Id
   --

   procedure W_Identifier (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Meta_Identifier
   --
   --    Next_Node                : Node_Id
   --    Identifier               : List_Id
   --

   procedure W_Meta_Identifier (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Terminal_String
   --
   --    Next_Node                : Node_Id
   --    The_String               : Name_Id
   --

   procedure W_Terminal_String (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Definitions_List
   --
   --    Next_Node                : Node_Id
   --    Single_Definition        : List_Id
   --

   procedure W_Definitions_List (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Single_Definition
   --
   --    Next_Node                : Node_Id
   --    Syntactic_Term           : List_Id
   --

   procedure W_Single_Definition (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Syntactic_Term
   --
   --    Next_Node                : Node_Id
   --    Syntactic_Factor         : Node_Id
   --    Syntactic_Exception      : Node_Id
   --

   procedure W_Syntactic_Term (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Syntactic_Exception
   --
   --    Next_Node                : Node_Id
   --    Syntactic_Factor         : Node_Id
   --

   procedure W_Syntactic_Exception (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Syntactic_Factor
   --
   --    Next_Node                : Node_Id
   --    Syntactic_Primary        : Node_Id
   --

   procedure W_Syntactic_Factor (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Syntactic_Primary
   --
   --    Next_Node                : Node_Id
   --    Element                  : Node_Id
   --

   procedure W_Syntactic_Primary (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Optional_Sequence
   --
   --    Next_Node                : Node_Id
   --    Definitions_List         : Node_Id
   --

   procedure W_Optional_Sequence (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Repeated_Sequence
   --
   --    Next_Node                : Node_Id
   --    Definitions_List         : Node_Id
   --

   procedure W_Repeated_Sequence (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Grouped_Sequence
   --
   --    Next_Node                : Node_Id
   --    Definitions_List         : Node_Id
   --

   procedure W_Grouped_Sequence (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Syntax_Rule
   --
   --    Next_Node                : Node_Id
   --    Meta_Identifier          : Node_Id
   --    Definitions_List         : Node_Id
   --

   procedure W_Syntax_Rule (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Syntax
   --
   --    Next_Node                : Node_Id
   --    Syntax_Rules             : List_Id
   --

   procedure W_Syntax (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   --
   --  Empty_Sequence
   --
   --    Next_Node                : Node_Id
   --

   procedure W_Empty_Sequence (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   function Kind (N : Types.Node_Id) return Node_Kind;
   --# global in State;
   --# pre N > Types.No_Node;

   procedure Set_Kind (N : Types.Node_Id; V : Node_Kind);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Loc (N : Types.Node_Id) return Locations.Location;
   --# global in State;
   --# pre N > Types.No_Node;

   procedure Set_Loc (N : Types.Node_Id; V : Locations.Location);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Next_Node (N : Types.Node_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;

   procedure Set_Next_Node (N : Types.Node_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function First_Node (N : Types.List_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_First_Node (N : Types.List_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Last_Node (N : Types.List_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Last_Node (N : Types.List_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Name (N : Types.Node_Id) return Types.Name_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Name;

   procedure Set_Name (N : Types.Node_Id; V : Types.Name_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Identifier (N : Types.Node_Id) return Types.List_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Identifier (N : Types.Node_Id; V : Types.List_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function The_String (N : Types.Node_Id) return Types.Name_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Name;

   procedure Set_The_String (N : Types.Node_Id; V : Types.Name_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Single_Definition (N : Types.Node_Id) return Types.List_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Single_Definition (N : Types.Node_Id; V : Types.List_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Syntactic_Term (N : Types.Node_Id) return Types.List_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Syntactic_Term (N : Types.Node_Id; V : Types.List_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Syntactic_Factor (N : Types.Node_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Syntactic_Factor (N : Types.Node_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Syntactic_Exception (N : Types.Node_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Syntactic_Exception (N : Types.Node_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Syntactic_Primary (N : Types.Node_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Syntactic_Primary (N : Types.Node_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function A_Element (N : Types.Node_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Element (N : Types.Node_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Definitions_List (N : Types.Node_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Definitions_List (N : Types.Node_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Meta_Identifier (N : Types.Node_Id) return Types.Node_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Meta_Identifier (N : Types.Node_Id; V : Types.Node_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   function Syntax_Rules (N : Types.Node_Id) return Types.List_Id;
   --# global in State;
   --# pre N > Types.No_Node;
   --# return Result => Result > Types.No_Node;

   procedure Set_Syntax_Rules (N : Types.Node_Id; V : Types.List_Id);
   --# global in out State;
   --# pre N > Types.No_Node;

   procedure W_Node (N : Types.Node_Id);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   Int_Array_Max : constant := 3;
   subtype Int_Array_Range is Integer range 1 .. Int_Array_Max;
   type Int_Array is array (Int_Array_Range) of Types.Int;

   type Node_Entry is record
      Kind : Node_Kind;
      L    : Int_Array;
      N    : Types.Name_Id;
      Loc  : Locations.Location;
   end record;

   Default_Node : constant Node_Entry :=
      Node_Entry'
     (Node_Kind'First,
      Int_Array'(Int_Array_Range => Types.Int'(0)),
      Types.No_Name,
      Locations.No_Location);

   procedure New_List
     (The_Kind : Node_Kind;
      The_Loc  : Locations.Location;
      Result   : out Types.List_Id);
   --# global in out State;
   --# post Result > Types.No_Node;

   procedure New_Node
     (The_Kind : Node_Kind;
      The_Loc  : Locations.Location;
      Result   : out Types.Node_Id);
   --# global in out State;
   --# post Result > Types.No_Node;

   procedure Append_Node_To_List (E : Types.Node_Id; L : Types.List_Id);
   --# global in out State;
   --# pre L > Types.No_Node;

   function Is_Empty (L : Types.List_Id) return Boolean;
   --# global in State;
   --  List manipulation

private

   N_Indents : Integer := -1;

   procedure W_Indents;
   --# global in     N_Indents;
   --#        in out Output.State;

   procedure W_List_Id (L : Types.List_Id);
   --# global in     State;
   --#        in out N_Indents;
   --#        in out Output.State;

   procedure W_Node_Id (N : Types.Node_Id);
   --# global in out N_Indents;
   --#        in out Output.State;

   procedure W_Node_Header (N : Types.Node_Id);
   --# global in     N_Indents;
   --#        in     State;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   procedure W_Node_Attribute
     (A : String;
      K : String;
      V : Spark.Strings.SparkString;
      N : Types.Int);
   --# global in     Output.Space_Increment;
   --#        in     State;
   --#        in out N_Indents;
   --#        in out Output.N_Space;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   function Node_Kind_Image
     (N    : Node_Kind)
      return Spark.Strings.SparkString;

   procedure Increment_Last;
   --# global in out State;

end EBNF.Nodes;
