------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                          E B N F . P A R S E R                           --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with EBNF.Lexer; use type EBNF.Lexer.Token_Type;
with Locations;
with Namet;
with EBNF.Nodes;
with Errors;

package body EBNF.Parser is

   --------------
   -- P_Tokens --
   --------------

   procedure P_Tokens (The_Tokens : Lexer.Token_List_Type) is
      Result : Boolean := False;
      Error_Token : Types.Name_Id;
   begin
      Lexer.Scan_Token (Fatal => True);

      for J in Lexer.Token_Range range The_Tokens'Range loop
         if The_Tokens (J) = Lexer.Token then
            Result := True;
         end if;
      end loop;

      if not Result then
         Errors.Set_Error_Loc (1, Lexer.Token_Location);
         Namet.Set_SparkStr_To_Name_Buffer
           (Lexer.Token_Type_Image (Lexer.Token));
         Namet.Name_Find (Error_Token);
         Errors.Set_Error_Name
           (1, Error_Token);
         Errors.Display_Error ("Wrong token: #");
      end if;
   end P_Tokens;

   ---------------------
   -- Found_Delimiter --
   ---------------------

   procedure Found_Delimiter
     (Delimiters : Lexer.Token_List_Type;
      Result     : out Boolean)
   is
      Escape : Boolean := False;
      Loc    : Locations.Location;
   begin
      Lexer.Save_Lexer (Loc);
      Lexer.Scan_Token (Fatal => True);

      for Index in Lexer.Token_Range range Delimiters'Range loop
         if Lexer.Token = (Delimiters (Index)) then
            Lexer.Restore_Lexer (Loc);
            Escape := True;
            exit;
         end if;
      end loop;

      Result := Escape;
   end Found_Delimiter;

   -----------------------
   -- P_Defining_Symbol --
   -----------------------

   --  defining symbol = '=';

   procedure P_Defining_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Defining_Symbol,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_Defining_Symbol;

   -------------------------
   -- P_Terminator_Symbol --
   -------------------------

   --  terminator symbol = ';' | '.';

   procedure P_Terminator_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Terminator_Symbol,
         2      => Lexer.T_Period,
         others => Lexer.T_EOF);

   begin
      P_Tokens (The_Tokens);
   end P_Terminator_Symbol;

   ---------------------
   -- P_Except_Symbol --
   ---------------------

   --  except symbol = '-';

   procedure P_Except_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Except_Symbol,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_Except_Symbol;

   --------------------------
   -- P_Start_Group_Symbol --
   --------------------------

   --  start group symbol = '(';

   procedure P_Start_Group_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Left_Parenthesis,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_Start_Group_Symbol;

   ------------------------
   -- P_End_Group_Symbol --
   ------------------------

   --  end group symbol = ')';

   procedure P_End_Group_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Right_Parenthesis,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_End_Group_Symbol;

   ---------------------------
   -- P_Start_Option_Symbol --
   ---------------------------

   --  start option symbol = '[' | '(/';
   --  XXX incomplete

   procedure P_Start_Option_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Left_Square_Bracket,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_Start_Option_Symbol;

   -------------------------
   -- P_End_Option_Symbol --
   -------------------------

   --  end option symbol = ']' | '/)';
   --  XXX incomplete

   procedure P_End_Option_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Right_Square_Bracket,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_End_Option_Symbol;

   ---------------------------
   -- P_Start_Repeat_Symbol --
   ---------------------------

   --  start repeat symbol = '{' | '(:';
   --  XXX incomplete

   procedure P_Start_Repeat_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Left_Curly_Bracket,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_Start_Repeat_Symbol;

   -------------------------
   -- P_End_Repeat_Symbol --
   -------------------------

   --  end repeat symbol = '}' | ':)';
   --  XXX incomplete

   procedure P_End_Repeat_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Right_Curly_Bracket,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_End_Repeat_Symbol;

   ---------------------------
   -- P_First_Quote_Symnbol --
   ---------------------------

   --  first quote symbol = "'";

   procedure P_First_Quote_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_First_Quote_Symbol,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_First_Quote_Symbol;

   ----------------------------
   -- P_Second_Quote_Symnbol --
   ----------------------------

   --  second quote symbol = '"';

   procedure P_Second_Quote_Symbol is
      The_Tokens : constant Lexer.Token_List_Type :=
         Lexer.Token_List_Type'
        (1      => Lexer.T_Second_Quote_Symbol,
         others => Lexer.T_EOF);
   begin
      P_Tokens (The_Tokens);
   end P_Second_Quote_Symbol;

   ------------------
   -- P_Identifier --
   ------------------

   procedure P_Identifier (Stack : in out Parser_Stack.Nodes_Stack) is
      Result_Node : Types.Node_Id;
      Loc         : Locations.Location;
   begin
      Lexer.Save_Lexer (Loc);
      Lexer.Scan_The_Token (Lexer.T_Identifier);
      if Lexer.Token = Lexer.T_Identifier then
         Nodes.New_Node (Nodes.K_Identifier, Loc, Result_Node);
         Nodes.Set_Name (Result_Node, Lexer.Token_Name);
         Parser_Stack.Push_Node (Stack, Result_Node);
      else
         Lexer.Restore_Lexer (Loc);
         Parser_Stack.Push (Parser_Stack.E_Error);
      end if;
   end P_Identifier;

   -----------------------
   -- P_Meta_Identifier --
   -----------------------

   --  (* see 4.14 *) meta identifier
   --    = letter, {meta identifier character};

   procedure P_Meta_Identifier is
   begin
      Parser_Stack.Push (Parser_Stack.E_Meta_Identifier_Epilogue);
      Parser_Stack.Push (Parser_Stack.E_Meta_Identifier_List_Of_Identifier);
      Parser_Stack.Push (Parser_Stack.E_Identifier);
      Parser_Stack.Push (Parser_Stack.E_Meta_Identifier_Prologue);
   end P_Meta_Identifier;

   --------------------------------
   -- P_Meta_Identifier_Prologue --
   --------------------------------

   procedure P_Meta_Identifier_Prologue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      Result_Node           : Types.Node_Id;
      Meta_Identifier_Rules : Types.List_Id;

   begin
      Nodes.New_Node
        (Nodes.K_Meta_Identifier,
         Lexer.Token_Location,
         Result_Node);
      Parser_Stack.Push_Node (Stack, Result_Node);

      Nodes.New_List
        (Nodes.K_List_Id,
         Lexer.Token_Location,
         Meta_Identifier_Rules);
      Parser_Stack.Push_Node (Stack, Meta_Identifier_Rules);
   end P_Meta_Identifier_Prologue;

   ----------------------------------------------------
   -- P_Meta_Identifier_List_Of_Meta_Identifier_Rule --
   ----------------------------------------------------

   procedure P_Meta_Identifier_List_Of_Meta_Identifier_Rule
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      List_Of_Identifiers : Types.List_Id;
      A_Identifier        : Types.Node_Id;
      Is_Delimiter_Found  : Boolean;
      Loc                 : Locations.Location;

   begin
      if Parser_Stack.Depth (Stack) >= 2 then
         Parser_Stack.Pop_Node (Stack, A_Identifier);
         Parser_Stack.Pop_Node (Stack, List_Of_Identifiers);

         if Types.Present (A_Identifier) then
            Nodes.Append_Node_To_List
              (A_Identifier,
               List_Of_Identifiers);
         end if;

         Lexer.Save_Lexer (Loc);
         Found_Delimiter
           (Lexer.Token_List_Type'(1      => Lexer.T_Defining_Symbol,
                                   2      => Lexer.T_Vertical_Line,
                                   3      => Lexer.T_Terminator_Symbol,
                                   4      => Lexer.T_Left_Curly_Bracket,
                                   5      => Lexer.T_Right_Curly_Bracket,
                                   6      => Lexer.T_Left_Square_Bracket,
                                   7      => Lexer.T_Right_Square_Bracket,
                                   8      => Lexer.T_Comma,
                                   9      => Lexer.T_Left_Parenthesis,
                                   10     => Lexer.T_Right_Parenthesis,
                                   11     => Lexer.T_Except_Symbol,
                                   others => Lexer.T_EOF),
            Is_Delimiter_Found);
         if not Is_Delimiter_Found then
            Lexer.Restore_Lexer (Loc);
            Parser_Stack.Push
              (Parser_Stack.E_Meta_Identifier_List_Of_Identifier);
            Parser_Stack.Push (Parser_Stack.E_Identifier);
         end if;

         Parser_Stack.Push_Node (Stack, List_Of_Identifiers);
      else
         null;
      end if;
   end P_Meta_Identifier_List_Of_Meta_Identifier_Rule;

   --------------------------------
   -- P_Meta_Identifier_Epilogue --
   --------------------------------

   procedure P_Meta_Identifier_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      Result_Node         : Types.Node_Id;
      List_Of_Identifiers : Types.List_Id;

   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, List_Of_Identifiers);

         if Types.Present (List_Of_Identifiers)
           and then not Nodes.Is_Empty (List_Of_Identifiers)
         then
            if Parser_Stack.Depth (Stack) >= 1 then
               Parser_Stack.Pop_Node (Stack, Result_Node);
               Nodes.Set_Loc
                 (List_Of_Identifiers,
                  Nodes.Loc (Nodes.First_Node (List_Of_Identifiers)));
               Nodes.Set_Identifier (Result_Node, List_Of_Identifiers);
               Parser_Stack.Push_Node (Stack, Result_Node);
            else
               null;
            end if;
         else
            Parser_Stack.Push (Parser_Stack.E_Error);
         end if;
      else
         null;
      end if;
   end P_Meta_Identifier_Epilogue;

   ------------------------
   -- P_Definitions_List --
   ------------------------

   --  (* see 4.4 *) definitions list
   --    = single definition,
   --      {definition separator symbol,
   --       single definition};

   procedure P_Definitions_List is
   begin
      Parser_Stack.Push (Parser_Stack.E_Definitions_List_Epilogue);
      Parser_Stack.Push
        (Parser_Stack.E_Definitions_List_List_Of_Single_Definition);
      Parser_Stack.Push (Parser_Stack.E_Single_Definition);
      Parser_Stack.Push (Parser_Stack.E_Definitions_List_Prologue);
   end P_Definitions_List;

   ---------------------------------
   -- P_Definitions_List_Prologue --
   ---------------------------------

   procedure P_Definitions_List_Prologue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Single_Definition : Types.List_Id;
      Result_Node           : Types.Node_Id;
   begin
      Nodes.New_Node
        (Nodes.K_Definitions_List,
         Lexer.Token_Location,
         Result_Node);
      Parser_Stack.Push_Node (Stack, Result_Node);

      Nodes.New_List
        (Nodes.K_List_Id,
         Lexer.Token_Location,
         The_Single_Definition);
      Parser_Stack.Push_Node (Stack, The_Single_Definition);
   end P_Definitions_List_Prologue;

   ---------------------------------
   -- P_Definitions_List_Epilogue --
   ---------------------------------

   procedure P_Definitions_List_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Single_Definition : Types.List_Id;
      Result_Node           : Types.Node_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 2 then
         Parser_Stack.Pop_Node (Stack, The_Single_Definition);
         Parser_Stack.Pop_Node (Stack, Result_Node);

         Nodes.Set_Single_Definition (Result_Node, The_Single_Definition);

         Parser_Stack.Push_Node (Stack, Result_Node);
      else
         null; --  XXX
      end if;
   end P_Definitions_List_Epilogue;

   --------------------------------------------------
   -- P_Definitions_List_List_Of_Single_Definition --
   --------------------------------------------------

   procedure P_Definitions_List_List_Of_Single_Definition
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      Separator : constant Lexer.Token_Type := Lexer.T_Vertical_Line;

      Loc   : Locations.Location;
      The_Single_Definition : Types.Node_Id;
      List_Of_Single_Definition : Types.List_Id;

   begin
      if Parser_Stack.Depth (Stack) >= 2 then
         Parser_Stack.Pop_Node (Stack, The_Single_Definition);
         Parser_Stack.Pop_Node (Stack, List_Of_Single_Definition);

         if Types.Present (The_Single_Definition) then
            Nodes.Append_Node_To_List (The_Single_Definition,
                                       List_Of_Single_Definition);
         end if;

         Lexer.Save_Lexer (Loc);
         Lexer.Scan_Token (Fatal => True);
         if Lexer.Token /= Separator then
            Lexer.Restore_Lexer (Loc);
         else
            Parser_Stack.Push
              (Parser_Stack.E_Definitions_List_List_Of_Single_Definition);
            Parser_Stack.Push (Parser_Stack.E_Single_Definition);
         end if;

         Parser_Stack.Push_Node (Stack, List_Of_Single_Definition);
      else
         null;
      end if;
   end P_Definitions_List_List_Of_Single_Definition;

   ------------------------
   -- P_Grouped_Sequence --
   ------------------------

   --  (* see 4.13 *) grouped sequence
   --    = start group symbol, definitions list,
   --      end group symbol;

   procedure P_Grouped_Sequence is
   begin
      Parser_Stack.Push (Parser_Stack.E_Grouped_Sequence_Epilogue);
      Parser_Stack.Push (Parser_Stack.E_End_Group_Symbol);
      Parser_Stack.Push (Parser_Stack.E_Definitions_List);
      Parser_Stack.Push (Parser_Stack.E_Start_Group_Symbol);
   end P_Grouped_Sequence;

   ---------------------------------
   -- P_Grouped_Sequence_Epilogue --
   ---------------------------------

   procedure P_Grouped_Sequence_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Definitions_List : Types.Node_Id;
      Result_Node          : Types.Node_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, The_Definitions_List);

         Nodes.New_Node
           (Nodes.K_Grouped_Sequence,
            Nodes.Loc (The_Definitions_List),
            Result_Node);
         Nodes.Set_Definitions_List (Result_Node, The_Definitions_List);
         Parser_Stack.Push_Node (Stack, Result_Node);
      else
         null;
      end if;
   end P_Grouped_Sequence_Epilogue;

   -------------------------
   -- P_Optional_Sequence --
   -------------------------

   --  (* see 4.11 *) optional sequence
   --    = start option symbol, definitions list,
   --      end option symbol;

   procedure P_Optional_Sequence is
   begin
      Parser_Stack.Push (Parser_Stack.E_Optional_Sequence_Epilogue);
      Parser_Stack.Push (Parser_Stack.E_End_Option_Symbol);
      Parser_Stack.Push (Parser_Stack.E_Definitions_List);
      Parser_Stack.Push (Parser_Stack.E_Start_Option_Symbol);
   end P_Optional_Sequence;

   ----------------------------------
   -- P_Optional_Sequence_Epilogue --
   ----------------------------------

   procedure P_Optional_Sequence_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Definitions_List : Types.Node_Id;
      Result_Node          : Types.Node_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, The_Definitions_List);

         Nodes.New_Node
           (Nodes.K_Optional_Sequence,
            Nodes.Loc (The_Definitions_List),
            Result_Node);
         Nodes.Set_Definitions_List (Result_Node, The_Definitions_List);

         Parser_Stack.Push_Node (Stack, Result_Node);
      else
         null;
      end if;
   end P_Optional_Sequence_Epilogue;

   -------------------------
   -- P_Repeated_Sequence --
   -------------------------

   --  (* see 4.12 *) repeated sequence
   --    = start repeat symbol, definitions list,
   --      end repeat symbol;

   procedure P_Repeated_Sequence is
   begin
      Parser_Stack.Push (Parser_Stack.E_Repeated_Sequence_Epilogue);
      Parser_Stack.Push (Parser_Stack.E_End_Repeat_Symbol);
      Parser_Stack.Push (Parser_Stack.E_Definitions_List);
      Parser_Stack.Push (Parser_Stack.E_Start_Repeat_Symbol);
   end P_Repeated_Sequence;

   ----------------------------------
   -- P_Repeated_Sequence_Epilogue --
   ----------------------------------

   procedure P_Repeated_Sequence_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Definitions_List : Types.Node_Id;
      Result_Node          : Types.Node_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, The_Definitions_List);

         Nodes.New_Node
           (Nodes.K_Repeated_Sequence,
            Nodes.Loc (The_Definitions_List),
         Result_Node);
         Nodes.Set_Definitions_List (Result_Node, The_Definitions_List);

         Parser_Stack.Push_Node (Stack, Result_Node);
      else
         null;
      end if;
   end P_Repeated_Sequence_Epilogue;

   -----------------------
   -- P_Terminal_String --
   -----------------------

   --  (* see 4.16 *) terminal string
   --    = first quote symbol, first terminal character,
   --      {first terminal character},
   --      first quote symbol
   --     | second quote symbol, second terminal character,
   --       {second terminal character},
   --       second quote symbol;

   procedure P_Terminal_String
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      Loc         : Locations.Location;
      T           : Lexer.Token_Type;
      Result_Node : Types.Node_Id;

   begin
      Lexer.Next_Token (T);
      case T is
         when Lexer.T_First_Quote_Symbol =>
            P_First_Quote_Symbol;
            Lexer.Save_Lexer (Loc);
            Lexer.Scan_The_Token (Lexer.T_First_String);
            if Lexer.Token = Lexer.T_First_String then
               Nodes.New_Node (Nodes.K_Terminal_String, Loc, Result_Node);
               Nodes.Set_The_String (Result_Node, Lexer.Token_Name);
               P_First_Quote_Symbol;
               Parser_Stack.Push_Node (Stack, Result_Node);
            else
               Lexer.Restore_Lexer (Loc);
               Parser_Stack.Push (Parser_Stack.E_Error);
            end if;

         when Lexer.T_Second_Quote_Symbol =>
            P_Second_Quote_Symbol;
            Lexer.Save_Lexer (Loc);
            Lexer.Scan_The_Token (Lexer.T_Second_String);
            if Lexer.Token = Lexer.T_Second_String then
               Nodes.New_Node (Nodes.K_Terminal_String, Loc, Result_Node);
               Nodes.Set_The_String (Result_Node, Lexer.Token_Name);
               P_Second_Quote_Symbol;
               Parser_Stack.Push_Node (Stack, Result_Node);
            else
               Lexer.Restore_Lexer (Loc);
               Parser_Stack.Push (Parser_Stack.E_Error);
            end if;

         when others =>
            Parser_Stack.Push (Parser_Stack.E_Error);
      end case;
   end P_Terminal_String;

   -------------------
   -- P_Syntax_Rule --
   -------------------

   --  (* see 4.3 *) syntax rule
   --    = meta identifier, defining symbol,
   --      definitions list, terminator symbol;

   procedure P_Syntax_Rule is
   begin
      Parser_Stack.Push (Parser_Stack.E_Syntax_Rule_Epilogue);
      Parser_Stack.Push (Parser_Stack.E_Terminator_Symbol);
      Parser_Stack.Push (Parser_Stack.E_Definitions_List);
      Parser_Stack.Push (Parser_Stack.E_Defining_Symbol);
      Parser_Stack.Push (Parser_Stack.E_Meta_Identifier);
   end P_Syntax_Rule;

   ----------------------------
   -- P_Syntax_Rule_Epilogue --
   ----------------------------

   procedure P_Syntax_Rule_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Meta_Identifier  : Types.Node_Id;
      The_Definitions_List : Types.Node_Id;
      Result_Node          : Types.Node_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 2 then
         Parser_Stack.Pop_Node (Stack, The_Definitions_List);
         Parser_Stack.Pop_Node (Stack, The_Meta_Identifier);

         Nodes.New_Node
           (Nodes.K_Syntax_Rule,
            Nodes.Loc (The_Meta_Identifier),
            Result_Node);
         Nodes.Set_Meta_Identifier (Result_Node, The_Meta_Identifier);
         Nodes.Set_Definitions_List (Result_Node, The_Definitions_List);

         Parser_Stack.Push_Node (Stack, Result_Node);
      else
         null;
      end if;
   end P_Syntax_Rule_Epilogue;

   --------------
   -- P_Syntax --
   --------------

   --  (* see 4.2 *) syntax
   --    = syntax rule, {syntax rule};

   procedure P_Syntax is
   begin
      Parser_Stack.Push (Parser_Stack.E_Syntax_Epilogue);
      Parser_Stack.Push (Parser_Stack.E_Syntax_List_Of_Syntax_Rule);
      Parser_Stack.Push (Parser_Stack.E_Syntax_Rule);
      Parser_Stack.Push (Parser_Stack.E_Syntax_Prologue);
   end P_Syntax;

   -----------------------
   -- P_Syntax_Prologue --
   -----------------------

   procedure P_Syntax_Prologue (Stack : in out Parser_Stack.Nodes_Stack) is
      Result_Node  : Types.Node_Id;
      Syntax_Rules : Types.List_Id;

   begin
      Nodes.New_Node (Nodes.K_Syntax, Lexer.Token_Location, Result_Node);
      Parser_Stack.Push_Node (Stack, Result_Node);

      Nodes.New_List (Nodes.K_List_Id, Lexer.Token_Location, Syntax_Rules);
      Parser_Stack.Push_Node (Stack, Syntax_Rules);
   end P_Syntax_Prologue;

   ----------------------------------
   -- P_Syntax_List_Of_Syntax_Rule --
   ----------------------------------

   procedure P_Syntax_List_Of_Syntax_Rule
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      Syntax_Rules       : Types.List_Id;
      A_Syntax_Rule      : Types.Node_Id;
      Is_Delimiter_Found : Boolean;
      Loc                : Locations.Location;

   begin
      if Parser_Stack.Depth (Stack) >= 2 then
         Parser_Stack.Pop_Node (Stack, A_Syntax_Rule);
         Parser_Stack.Pop_Node (Stack, Syntax_Rules);

         if Types.Present (A_Syntax_Rule) then
            Nodes.Append_Node_To_List (A_Syntax_Rule, Syntax_Rules);
         end if;

         Lexer.Save_Lexer (Loc);
         Found_Delimiter
           (Lexer.Token_List_Type'(others => Lexer.T_EOF),
            Is_Delimiter_Found);
         if not Is_Delimiter_Found then
            Parser_Stack.Push (Parser_Stack.E_Syntax_List_Of_Syntax_Rule);
            Parser_Stack.Push (Parser_Stack.E_Syntax_Rule);
            Lexer.Restore_Lexer (Loc);
         end if;

         Parser_Stack.Push_Node (Stack, Syntax_Rules);
      else
         null;
      end if;
   end P_Syntax_List_Of_Syntax_Rule;

   -----------------------
   -- P_Syntax_Epilogue --
   -----------------------

   procedure P_Syntax_Epilogue (Stack : in out Parser_Stack.Nodes_Stack) is
      Result_Node  : Types.Node_Id;
      Syntax_Rules : Types.List_Id;

   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, Syntax_Rules);
         if Types.Present (Syntax_Rules) then
            if Parser_Stack.Depth (Stack) >= 1 then
               Parser_Stack.Pop_Node (Stack, Result_Node);
               Nodes.Set_Loc
                 (Syntax_Rules,
                  Nodes.Loc (Nodes.First_Node (Syntax_Rules)));
               Nodes.Set_Syntax_Rules (Result_Node, Syntax_Rules);
               Parser_Stack.Push_Node (Stack, Result_Node);
            else
               null;
            end if;

         else
            Parser_Stack.Push (Parser_Stack.E_Error);
         end if;
      else
         null;
      end if;
   end P_Syntax_Epilogue;

   ----------------------
   -- P_Empty_Sequence --
   ----------------------

   --  (* see 4.21 *) empty sequence
   --    = ;

   procedure P_Empty_Sequence (Stack : in out Parser_Stack.Nodes_Stack) is
      Result_Node : Types.Node_Id;
   begin
      Nodes.New_Node
        (Nodes.K_Empty_Sequence,
         Lexer.Token_Location,
         Result_Node);

      Parser_Stack.Push_Node (Stack, Result_Node);
   end P_Empty_Sequence;

   -------------------------
   -- P_Syntactic_Primary --
   -------------------------

   --  (* see 4.10 *) syntactic primary
   --    = optional sequence
   --      | repeated sequence
   --      | grouped sequence
   --      | meta identifier
   --      | terminal string
   --      | special sequence
   --      | empty sequence;

   procedure P_Syntactic_Primary is
      T : Lexer.Token_Type;
   begin
      Parser_Stack.Push (Parser_Stack.E_Syntactic_Primary_Epilogue);

      Lexer.Next_Token (T);
      case T is
         when Lexer.T_Left_Square_Bracket =>
            Parser_Stack.Push (Parser_Stack.E_Optional_Sequence);

         when Lexer.T_Left_Curly_Bracket =>
            Parser_Stack.Push (Parser_Stack.E_Repeated_Sequence);

         when Lexer.T_Left_Parenthesis =>
            Parser_Stack.Push (Parser_Stack.E_Grouped_Sequence);

         when Lexer.T_Identifier =>
            Parser_Stack.Push (Parser_Stack.E_Meta_Identifier);

         when Lexer.T_First_Quote_Symbol | Lexer.T_Second_Quote_Symbol =>
            Parser_Stack.Push (Parser_Stack.E_Terminal_String);

         when Lexer.T_Terminator_Symbol =>
            Parser_Stack.Push (Parser_Stack.E_Empty_Sequence);

         when others =>
            Parser_Stack.Push (Parser_Stack.E_Error);
      end case;
   end P_Syntactic_Primary;

   ----------------------------------
   -- P_Syntactic_Primary_Epilogue --
   ----------------------------------

   procedure P_Syntactic_Primary_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      N           : Types.Node_Id;
      Result_Node : Types.Node_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, N);
         if Types.No (N) then
            Result_Node := Types.No_Node;
         else
            Nodes.New_Node
              (Nodes.K_Syntactic_Primary,
               Nodes.Loc (N),
               Result_Node);
            Nodes.Set_Element (Result_Node, N);
         end if;
         Parser_Stack.Push_Node (Stack, Result_Node);
      else
         null;
      end if;
   end P_Syntactic_Primary_Epilogue;

   -------------------------
   -- P_Single_Definition --
   -------------------------

   --  (* see 4.5 *) single definition
   --    = syntactic term,
   --      {concatenate symbol, syntactic term};

   procedure P_Single_Definition is
   begin
      Parser_Stack.Push (Parser_Stack.E_Single_Definition_Epilogue);
      Parser_Stack.Push
        (Parser_Stack.E_Single_Definition_List_Of_Syntactic_Term);
      Parser_Stack.Push (Parser_Stack.E_Syntactic_Term);
      Parser_Stack.Push (Parser_Stack.E_Single_Definition_Prologue);
   end P_Single_Definition;

   ----------------------------------
   -- P_Single_Definition_Prologue --
   ----------------------------------

   procedure P_Single_Definition_Prologue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Single_Definition   : Types.Node_Id;
      List_Of_Syntactic_Terms : Types.List_Id;
   begin
      Nodes.New_Node
        (Nodes.K_Single_Definition,
         Lexer.Token_Location,
         The_Single_Definition);
      Parser_Stack.Push_Node (Stack, The_Single_Definition);

      Nodes.New_List
        (Nodes.K_List_Id,
         Lexer.Token_Location,
         List_Of_Syntactic_Terms);
      Parser_Stack.Push_Node (Stack, List_Of_Syntactic_Terms);
   end P_Single_Definition_Prologue;

   ------------------------------------------------
   -- P_Single_Definition_List_Of_Syntactic_Term --
   ------------------------------------------------

   procedure P_Single_Definition_List_Of_Syntactic_Term
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      Separator : constant Lexer.Token_Type := Lexer.T_Comma;

      The_Syntactic_Term      : Types.Node_Id;
      List_Of_Syntactic_Terms : Types.List_Id;
      Loc                     : Locations.Location;
   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, The_Syntactic_Term);

         if Types.Present (The_Syntactic_Term) then
            if Parser_Stack.Depth (Stack) >= 1 then
               Parser_Stack.Pop_Node (Stack, List_Of_Syntactic_Terms);
               --   XXX Test should always be true
               Nodes.Append_Node_To_List
                 (The_Syntactic_Term,
                  List_Of_Syntactic_Terms);

               Lexer.Save_Lexer (Loc);
               Lexer.Scan_Token (Fatal => True);
               if Lexer.Token /= Separator then
                  Lexer.Restore_Lexer (Loc);
               else
                  Parser_Stack.Push
                    (Parser_Stack.E_Single_Definition_List_Of_Syntactic_Term);
                  Parser_Stack.Push (Parser_Stack.E_Syntactic_Term);
               end if;

               Parser_Stack.Push_Node (Stack, List_Of_Syntactic_Terms);

            else
               null;
            end if;
         end if;

      else
         null;
      end if;
   end P_Single_Definition_List_Of_Syntactic_Term;

   ----------------------------------
   -- P_Single_Definition_Epilogue --
   ----------------------------------

   procedure P_Single_Definition_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Single_Definition   : Types.Node_Id;
      List_Of_Syntactic_Terms : Types.List_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 2 then
         Parser_Stack.Pop_Node (Stack, List_Of_Syntactic_Terms);
         Parser_Stack.Pop_Node (Stack, The_Single_Definition);

         Nodes.Set_Syntactic_Term
           (The_Single_Definition,
            List_Of_Syntactic_Terms);

         Parser_Stack.Push_Node (Stack, The_Single_Definition);
      else
         null;
      end if;
   end P_Single_Definition_Epilogue;

   ------------------------
   -- P_Syntactic_Factor --
   ------------------------

   --  (* see 4.8 *) syntactic factor
   --    = [integer, repetition symbol],
   --      syntactic primary;
   --  XXX incomplete

   procedure P_Syntactic_Factor is
   begin
      Parser_Stack.Push (Parser_Stack.E_Syntactic_Factor_Epilogue);
      Parser_Stack.Push (Parser_Stack.E_Syntactic_Primary);
   end P_Syntactic_Factor;

   ----------------------------------
   --  P_Syntactic_Factor_Epilogue --
   ----------------------------------

   procedure P_Syntactic_Factor_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Syntactic_Primary : Types.Node_Id;
      Result_Node           : Types.Node_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, The_Syntactic_Primary);

         Nodes.New_Node
           (Nodes.K_Syntactic_Factor,
            Nodes.Loc (The_Syntactic_Primary),
            Result_Node);
         Nodes.Set_Syntactic_Primary (Result_Node, The_Syntactic_Primary);

         Parser_Stack.Push_Node (Stack, Result_Node);
      else
         null;
      end if;
   end P_Syntactic_Factor_Epilogue;

   ---------------------------
   -- P_Syntactic_Exception --
   ---------------------------

   --  (* see 4.7 *) syntactic exception
   --   = syntactic factor;

   procedure P_Syntactic_Exception (Stack : in out Parser_Stack.Nodes_Stack) is
      The_Syntactic_Factor : Types.Node_Id;
      Result_Node : Types.Node_Id;
      The_Syntactic_Term : Types.Node_Id;
   begin
      if Parser_Stack.Depth (Stack) >= 2 then
         Parser_Stack.Pop_Node (Stack, The_Syntactic_Factor);
         Parser_Stack.Pop_Node (Stack, The_Syntactic_Term);

         Nodes.New_Node
           (Nodes.K_Syntactic_Exception,
            Nodes.Loc (The_Syntactic_Factor),
            Result_Node);
         Nodes.Set_Syntactic_Factor (Result_Node, The_Syntactic_Factor);

         Nodes.Set_Syntactic_Exception (The_Syntactic_Term, Result_Node);
         Parser_Stack.Push_Node (Stack, The_Syntactic_Term);
      else
         null;
      end if;
   end P_Syntactic_Exception;

   ----------------------
   -- P_Syntactic_Term --
   ----------------------

   --  (* see 4.6 *) syntactic term
   --    = syntactic factor,
   --      [except symbol, syntactic exception];

   procedure P_Syntactic_Term is
   begin
      Parser_Stack.Push (Parser_Stack.E_Syntactic_Term_Epilogue);
      Parser_Stack.Push (Parser_Stack.E_Syntactic_Factor);
   end P_Syntactic_Term;

   -------------------------------
   -- P_Syntactic_Term_Epilogue --
   -------------------------------

   procedure P_Syntactic_Term_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack)
   is
      The_Syntactic_Factor : Types.Node_Id;
      Result_Node          : Types.Node_Id;
      The_Token : Lexer.Token_Type;

   begin
      if Parser_Stack.Depth (Stack) >= 1 then
         Parser_Stack.Pop_Node (Stack, The_Syntactic_Factor);

         Nodes.New_Node
           (Nodes.K_Syntactic_Term,
            Nodes.Loc (The_Syntactic_Factor),
            Result_Node);
         Nodes.Set_Syntactic_Factor (Result_Node, The_Syntactic_Factor);

         Parser_Stack.Push_Node (Stack, Result_Node);

         Lexer.Next_Token (The_Token);
         if The_Token = Lexer.T_Except_Symbol then
            P_Except_Symbol;
            Parser_Stack.Push (Parser_Stack.E_Syntactic_Exception);
            Parser_Stack.Push (Parser_Stack.E_Syntactic_Factor);

            --  P_Syntactic_Exception (The_Syntactic_Exception);
            --  Nodes.Set_Syntactic_Exception (Result_Node,
            --                                 The_Syntactic_Exception);
         end if;
      else
         null;
      end if;
   end P_Syntactic_Term_Epilogue;

   -----------
   -- Parse --
   -----------

   procedure Parse (Result_Node : out Types.Node_Id) is
      Node_Element_Kind : Parser_Stack.Element_Kind_Type;
      Error_Detected    : Boolean := False;
      Stack : Parser_Stack.Nodes_Stack;

   begin
      Parser_Stack.Initialize (Stack);

      Parser_Stack.Push (Parser_Stack.E_Syntax);

      --# assert Parser_Stack.Valid_Stack (Stack);
      loop
         --# assert Parser_Stack.Valid_Stack (Stack);
         exit when Error_Detected or else Parser_Stack.Is_Empty;
         Parser_Stack.Pop (Node_Element_Kind);
         --# check Parser_Stack.Valid_Stack (Stack);

         case Node_Element_Kind is

         --  'Syntax' production rule

         when Parser_Stack.E_Syntax =>
            P_Syntax;

         when Parser_Stack.E_Syntax_Prologue =>
            P_Syntax_Prologue (Stack);

         when Parser_Stack.E_Syntax_Epilogue =>
            P_Syntax_Epilogue (Stack);

         when Parser_Stack.E_Syntax_List_Of_Syntax_Rule =>
            P_Syntax_List_Of_Syntax_Rule (Stack);

         --  'Syntax rule' production rules

         when Parser_Stack.E_Syntax_Rule =>
            P_Syntax_Rule;

         when Parser_Stack.E_Syntax_Rule_Epilogue =>
            P_Syntax_Rule_Epilogue (Stack);

         --  'Syntactic primary' production rules

         when Parser_Stack.E_Syntactic_Primary =>
            P_Syntactic_Primary;

         when Parser_Stack.E_Syntactic_Primary_Epilogue =>
            P_Syntactic_Primary_Epilogue (Stack);

         when Parser_Stack.E_Syntactic_Factor =>
            P_Syntactic_Factor;

         when Parser_Stack.E_Syntactic_Factor_Epilogue =>
            P_Syntactic_Factor_Epilogue (Stack);

         when Parser_Stack.E_Syntactic_Term =>
            P_Syntactic_Term;

         when Parser_Stack.E_Syntactic_Term_Epilogue =>
            P_Syntactic_Term_Epilogue (Stack);

         when Parser_Stack.E_Syntactic_Exception =>
            P_Syntactic_Exception (Stack);

         --  'Meta identifier' production rule

         when Parser_Stack.E_Meta_Identifier =>
            P_Meta_Identifier;

         when Parser_Stack.E_Identifier =>
            P_Identifier (Stack);

         when Parser_Stack.E_Meta_Identifier_Epilogue =>
            P_Meta_Identifier_Epilogue (Stack);

         when Parser_Stack.E_Meta_Identifier_List_Of_Identifier =>
            P_Meta_Identifier_List_Of_Meta_Identifier_Rule (Stack);

         when Parser_Stack.E_Meta_Identifier_Prologue =>
            P_Meta_Identifier_Prologue (Stack);

         --  'Definitions list' production rule

         when Parser_Stack.E_Definitions_List =>
            P_Definitions_List;

         when Parser_Stack.E_Definitions_List_List_Of_Single_Definition =>
            P_Definitions_List_List_Of_Single_Definition (Stack);

         when Parser_Stack.E_Definitions_List_Epilogue =>
            P_Definitions_List_Epilogue (Stack);

         when Parser_Stack.E_Definitions_List_Prologue =>
            P_Definitions_List_Prologue (Stack);

         --  'single definition' production rule

         when Parser_Stack.E_Single_Definition =>
            P_Single_Definition;

         when Parser_Stack.E_Single_Definition_Epilogue =>
            P_Single_Definition_Epilogue (Stack);

         when Parser_Stack.E_Single_Definition_List_Of_Syntactic_Term =>
            P_Single_Definition_List_Of_Syntactic_Term (Stack);

         when Parser_Stack.E_Single_Definition_Prologue =>
            P_Single_Definition_Prologue (Stack);

         --  'grouped sequence' production rule

         when Parser_Stack.E_Grouped_Sequence =>
            P_Grouped_Sequence;

         when Parser_Stack.E_Grouped_Sequence_Epilogue =>
            P_Grouped_Sequence_Epilogue (Stack);

         --  'repeated sequence' production rule

         when Parser_Stack.E_Repeated_Sequence =>
            P_Repeated_Sequence;

         when Parser_Stack.E_Repeated_Sequence_Epilogue =>
            P_Repeated_Sequence_Epilogue (Stack);

         --  'optional sequence' production rule

         when Parser_Stack.E_Optional_Sequence =>
            P_Optional_Sequence;

         when Parser_Stack.E_Optional_Sequence_Epilogue =>
            P_Optional_Sequence_Epilogue (Stack);

         --  'empty sequence' production rule

         when Parser_Stack.E_Empty_Sequence =>
            P_Empty_Sequence (Stack);

         when Parser_Stack.E_Terminal_String =>
            P_Terminal_String (Stack);

         --  Terminals production rules

         when Parser_Stack.E_Start_Group_Symbol =>
            P_Start_Group_Symbol;

         when Parser_Stack.E_End_Group_Symbol =>
            P_End_Group_Symbol;

         when Parser_Stack.E_Start_Option_Symbol =>
            P_Start_Option_Symbol;

         when Parser_Stack.E_End_Option_Symbol =>
            P_End_Option_Symbol;

         when Parser_Stack.E_Start_Repeat_Symbol =>
            P_Start_Repeat_Symbol;

         when Parser_Stack.E_End_Repeat_Symbol =>
            P_End_Repeat_Symbol;

         when Parser_Stack.E_Defining_Symbol =>
            P_Defining_Symbol;

         when Parser_Stack.E_Terminator_Symbol =>
            P_Terminator_Symbol;

         --  Management of error

         when Parser_Stack.E_Error =>
            Error_Detected := True;

         when Parser_Stack.E_No_Element =>
            Error_Detected := True;
            Errors.Set_Error_Loc (1, Lexer.Token_Location);
            Errors.Display_Error ("ICE :(");

         end case;
      end loop;
      --# assert Parser_Stack.Valid_Stack (Stack);

      if Error_Detected
        or else Parser_Stack.Is_EmptyT (Stack)
      then
         Result_Node := Types.No_Node;
      else
         --# accept F, 10, Stack,
         --#  "Expected ineffective assignment to Stack";
         Parser_Stack.Pop_Node (Stack, Result_Node);
      end if;
   end Parse;

end EBNF.Parser;
