------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                          E B N F . P A R S E R                           --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Types;
with EBNF.Lexer;
with EBNF.Parser_Stack;

--# inherit EBNF.Lexer,
--#         EBNF.Nodes,
--#         EBNF.Parser_Stack,
--#         Errors,
--#         Locations,
--#         Namet,
--#         Output,
--#         Types;

package EBNF.Parser is

   procedure Parse (Result_Node : out Types.Node_Id);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Nodes.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;

private

   procedure P_Syntax;
   --# global in out Parser_Stack.State;

   procedure P_Syntax_Epilogue (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Syntax_List_Of_Syntax_Rule
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Nodes.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Syntax_Prologue (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Token_Location;
   --#        in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Tokens (The_Tokens : Lexer.Token_List_Type);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure Found_Delimiter
     (Delimiters : Lexer.Token_List_Type;
      Result     : out Boolean);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_Defining_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_Terminator_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_Except_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_Start_Group_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_End_Group_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_Start_Option_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_End_Option_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_Start_Repeat_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_End_Repeat_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_First_Quote_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_Second_Quote_Symbol;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure P_Identifier (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Nodes.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Meta_Identifier;
   --# global in out Parser_Stack.State;

   procedure P_Meta_Identifier_Prologue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Token_Location;
   --#        in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Meta_Identifier_List_Of_Meta_Identifier_Rule
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Nodes.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Meta_Identifier_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Definitions_List;
   --# global in out Parser_Stack.State;

   procedure P_Definitions_List_Prologue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Token_Location;
   --#        in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Definitions_List_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Definitions_List_List_Of_Single_Definition
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Nodes.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Single_Definition;
   --# global in out Parser_Stack.State;

   procedure P_Single_Definition_Prologue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Token_Location;
   --#        in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Single_Definition_List_Of_Syntactic_Term
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Nodes.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Single_Definition_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Syntactic_Factor;
   --# global in out Parser_Stack.State;

   procedure P_Syntactic_Factor_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Syntactic_Primary;
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;

   procedure P_Syntactic_Primary_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Syntactic_Term;
   --# global in out Parser_Stack.State;

   procedure P_Syntactic_Term_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Nodes.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Terminal_String
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Buffer;
   --#        in     Lexer.Token_Image;
   --#        in out Errors.Error_Int;
   --#        in out Errors.Error_Loc;
   --#        in out Errors.Error_Name;
   --#        in out Errors.N_Errors;
   --#        in out Lexer.Token;
   --#        in out Lexer.Token_Location;
   --#        in out Lexer.Token_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Nodes.State;
   --#        in out Output.State;
   --#        in out Parser_Stack.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Syntax_Rule;
   --# global in out Parser_Stack.State;

   procedure P_Syntax_Rule_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Grouped_Sequence;
   --# global in out Parser_Stack.State;

   procedure P_Grouped_Sequence_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Optional_Sequence;
   --# global in out Parser_Stack.State;

   procedure P_Optional_Sequence_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Repeated_Sequence;
   --# global in out Parser_Stack.State;

   procedure P_Repeated_Sequence_Epilogue
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Empty_Sequence
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in     Lexer.Token_Location;
   --#        in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

   procedure P_Syntactic_Exception
     (Stack : in out Parser_Stack.Nodes_Stack);
   --# global in out Nodes.State;
   --# pre Parser_Stack.Valid_Stack (Stack);
   --# post Parser_Stack.Valid_Stack (Stack);

end EBNF.Parser;
