------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                    E B N F . P A R S E R _ S T A C K                     --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

package body EBNF.Parser_Stack
--# own State is The_Stack, Stack_Pointer;
is

   The_Stack     : Stack_Type         :=
     Stack_Type'(Index => Null_Element);
   Stack_Pointer : Stack_Pointer_Type := 0;

   --------------
   -- Is_Empty --
   --------------

   function Is_Empty return Boolean
   --# global in Stack_Pointer;
   --# return Stack_Pointer = 0;
   is
   begin
      return Stack_Pointer = 0;
   end Is_Empty;

   ----------
   -- Push --
   ----------

   procedure Push (Element_Kind : Element_Kind_Type)
   --# global in out Stack_Pointer;
   --#        in out The_Stack;
   is
   begin
      if Stack_Pointer < The_Stack'Last then
         Stack_Pointer             := Stack_Pointer + 1;
         The_Stack (Stack_Pointer) :=
           Stack_Element'(Element_Kind => Element_Kind);
      end if;
   end Push;

   -----------
   -- Depth --
   -----------

   function Depth (T : Nodes_Stack) return Stack_Pointer_Type is
   begin
      return T.Node_Stack_Pointer;
   end Depth;

   ---------
   -- Pop --
   ---------

   procedure Pop (Element_Kind : out Element_Kind_Type)
   --# global in     The_Stack;
   --#        in out Stack_Pointer;
   is
   begin
      if Stack_Pointer = 0 then
         Element_Kind := E_No_Element;

      else
         Element_Kind  := The_Stack (Stack_Pointer).Element_Kind;
         Stack_Pointer := Stack_Pointer - 1;
      end if;
   end Pop;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize (T : out Nodes_Stack) is
   begin
      T := Nodes_Stack'
     (Node_Stack_Type'
        (Index => Node_Stack_Element'(Node => Types.No_Node)),
         0);
   end Initialize;

   --------------
   -- Is_Empty --
   --------------

   function Is_EmptyT (T : Nodes_Stack) return Boolean is
   begin
      return T.Node_Stack_Pointer = 0;
   end Is_EmptyT;

   ---------------
   -- Push_Node --
   ---------------

   procedure Push_Node (T : in out Nodes_Stack; Node : Types.Node_Id) is
   begin
      if T.Node_Stack_Pointer < T.Node_Stack'Last then
         T.Node_Stack_Pointer              := T.Node_Stack_Pointer + 1;
         T.Node_Stack (T.Node_Stack_Pointer) :=
           Node_Stack_Element'(Node => Node);
      end if;
   end Push_Node;

   --------------
   -- Pop_Node --
   --------------

   procedure Pop_Node (T : in out Nodes_Stack; Node : out Types.Node_Id) is
   begin
      Node                 := T.Node_Stack (T.Node_Stack_Pointer).Node;
      T.Node_Stack_Pointer := T.Node_Stack_Pointer - 1;
   end Pop_Node;

end EBNF.Parser_Stack;
