------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                    E B N F . P A R S E R _ S T A C K                     --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Types;

--# inherit Types;
package EBNF.Parser_Stack
--# own State;
--# initializes State;
is

   type Element_Kind_Type is
     (E_No_Element,

      --  Sub-states for parsing 'syntax' elements
      E_Syntax,
      E_Syntax_Epilogue,
      E_Syntax_List_Of_Syntax_Rule,
      E_Syntax_Prologue,

      --  Sub-state for parsing 'syntax rules"
      E_Syntax_Rule,
      E_Syntax_Rule_Epilogue,

      --  Sub-state for parsing 'syntactic primary'
      E_Syntactic_Primary,
      E_Syntactic_Primary_Epilogue,

      --  Sub-state for parsing 'syntactic factor'
      E_Syntactic_Factor,
      E_Syntactic_Factor_Epilogue,

      --  Sub-state for parsing 'syntactic term'
      E_Syntactic_Term,
      E_Syntactic_Term_Epilogue,

      --  Sub-state for parsing 'syntactic exception'
      E_Syntactic_Exception,

      --  Sub-state for parsing 'meta identifier'
      E_Meta_Identifier,
      E_Identifier,
      E_Meta_Identifier_Epilogue,
      E_Meta_Identifier_List_Of_Identifier,
      E_Meta_Identifier_Prologue,

      --  Sub-state for parsing 'definition list'
      E_Definitions_List,
      E_Definitions_List_Epilogue,
      E_Definitions_List_List_Of_Single_Definition,
         E_Definitions_List_Prologue,

      --  Sub-state for parsing 'single definition'
      E_Single_Definition,
      E_Single_Definition_Epilogue,
      E_Single_Definition_List_Of_Syntactic_Term,
      E_Single_Definition_Prologue,

      --  Sub-state for parsing 'grouped sequence'
      E_Grouped_Sequence,
      E_Grouped_Sequence_Epilogue,

      --  Sub-state for parsing 'repeated sequence'
      E_Repeated_Sequence,
      E_Repeated_Sequence_Epilogue,

      --  Sub-state for parsing 'optional sequence'
      E_Optional_Sequence,
      E_Optional_Sequence_Epilogue,

      --  Sub-state for parsing 'empty sequence'
      E_Empty_Sequence,

      --  Sub-state for parsing 'terminal string'
      E_Terminal_String,

      --  Sub-state for terminals
      E_Start_Group_Symbol,
      E_End_Group_Symbol,
      E_Start_Option_Symbol,
      E_End_Option_Symbol,
      E_Start_Repeat_Symbol,
      E_End_Repeat_Symbol,
      E_Terminator_Symbol,
      E_Defining_Symbol,

      --  Sub-state denoting an error
      E_Error);

   function Is_Empty return Boolean;
   --# global in State;

   procedure Push (Element_Kind : Element_Kind_Type);
   --# global in out State;

   procedure Pop (Element_Kind : out Element_Kind_Type);
   --# global in out State;

   --   type Nodes_Stack is private;
   Stack_Size : constant := 1_000;
   subtype Stack_Pointer_Type is Integer range 0 .. Stack_Size;

   subtype Index is Integer range 1 .. Stack_Pointer_Type'Last;

   type Node_Stack_Element is record
      Node : Types.Node_Id;
   end record;

   type Node_Stack_Type is array (Index) of Node_Stack_Element;

   type Nodes_Stack is record
      Node_Stack         : Node_Stack_Type;
      Node_Stack_Pointer : Stack_Pointer_Type;
   end record;
   Empty_Stack : constant Nodes_Stack;

   --# function Valid_Stack (S : Nodes_Stack) return Boolean;
   --  Valid_Stack (S) :=
   --    J <= S.Node_Stack_Pointer => S.Node_Stack (J).Node > Types.No_Node

   procedure Initialize (T : out Nodes_Stack);
   --# post Valid_Stack (T);

   function Is_EmptyT (T : Nodes_Stack) return Boolean;
   --# return T.Node_Stack_Pointer = 0;

   function Depth (T : Nodes_Stack) return Stack_Pointer_Type;
   --# return T.Node_Stack_Pointer;

   procedure Push_Node (T : in out Nodes_Stack; Node : Types.Node_Id);
   --# pre Node > Types.No_Node
   --# and Valid_Stack (T);
   --# post (T~.Node_Stack_Pointer = Node_Stack_Type'Last
   --#       or T.Node_Stack_Pointer = T~.Node_Stack_Pointer + 1)
   --# and Valid_Stack (T);

   procedure Pop_Node (T : in out Nodes_Stack; Node : out Types.Node_Id);
   --# pre T.Node_Stack_Pointer > 0
   --#  and Valid_Stack (T);
   --# post Node > Types.No_Node
   --# and T.Node_Stack_Pointer = T~.Node_Stack_Pointer - 1
   --# and Valid_Stack (T);

   --  XXX pre should be equivalent to pre not Is_Emptyt (T)

private
   type Stack_Element is record
      Element_Kind : Element_Kind_Type;
   end record;

   Null_Element : constant Stack_Element :=
     Stack_Element'(Element_Kind => E_No_Element);

   type Stack_Type is array (Index) of Stack_Element;

   Empty_Stack : constant Nodes_Stack := Nodes_Stack'
     (Node_Stack_Type'
        (Index => Node_Stack_Element'(Node => Types.No_Node)),
      0);

end EBNF.Parser_Stack;
