------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                         E B N F . P R I N T E R                          --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Types;
--# inherit EBNF.Lexer,
--#         EBNF.Nodes,
--#         Namet,
--#         Output,
--#         Types;

package EBNF.Printer
   --# own Stack_State;
   --# initializes Stack_State;
is

   procedure Write (N : Types.Node_Id);
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in     Nodes.State;
   --#        in out Output.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

private

   procedure W_Definition_Separator_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_Concatenate_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_Defining_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_Terminator_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_Start_Group_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_End_Group_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_Start_Option_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_End_Option_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_Start_Repeat_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_End_Repeat_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_First_Quote_Symbol;
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in out Output.State;

   procedure W_Identifier (N : Types.Node_Id);
   --# global in     Namet.State;
   --#        in     Nodes.State;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   procedure W_Meta_Identifier (N : Types.Node_Id);
   --# global in     Namet.State;
   --#        in     Nodes.State;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   procedure W_Definitions_List (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Single_Definition (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Syntactic_Factor (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Syntactic_Primary (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Syntactic_Term (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Terminal_String (N : Types.Node_Id);
   --# global in     Lexer.Token_Image;
   --#        in     Namet.State;
   --#        in     Nodes.State;
   --#        in out Output.State;
   --# pre N > Types.No_Node;

   procedure W_Syntax_Rule (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Grouped_Sequence (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Optional_Sequence (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Repeated_Sequence (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   procedure W_Empty_Sequence;

   procedure W_Syntax (N : Types.Node_Id);
   --# global in     Nodes.State;
   --#        in out Stack_State;
   --# pre N > Types.No_Node;

   --     procedure W_Syntactic_Exception (N : Types.Node_Id);

end EBNF.Printer;
