------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                             E B N F 2 I D L                              --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Spark.Command_Line;

with EBNF.Nodes;    use type EBNF.Nodes.Node_Kind;
with EBNF.Printer;

with Output;
with Namet;
with Errors;
with Types;         use type Types.Int, Types.Name_Id, Types.Byte;

package body EBNF2IDL is
   --# hide EBNF2IDL

   -----------------
   -- Is_Terminal --
   -----------------

   function Is_Terminal (N : Types.Node_Id) return Boolean is
      pragma Assert (EBNF.Nodes.Kind (N) = EBNF.Nodes.K_Syntax_Rule);

      I, J : Types.Node_Id;
      The_Syntactic_Primary : Types.Node_Id;
      The_Definitions_List : constant Types.Node_Id
        := EBNF.Nodes.Definitions_List (N);

      Rule_Is_Terminal : Boolean := True;

   begin
      J := EBNF.Nodes.First_Node
        (EBNF.Nodes.Single_Definition (The_Definitions_List));

      while J /= Types.No_Node loop
         I := EBNF.Nodes.First_Node (EBNF.Nodes.Syntactic_Term (J));
         while I /= Types.No_Node loop
            The_Syntactic_Primary := EBNF.Nodes.Syntactic_Primary
              (EBNF.Nodes.Syntactic_Factor (I));

            if EBNF.Nodes.Kind (EBNF.Nodes.A_Element (The_Syntactic_Primary))
              /= EBNF.Nodes.K_Terminal_String
            then
               Rule_Is_Terminal := False;
            end if;

            I := EBNF.Nodes.Next_Node (I);
         end loop;
         J := EBNF.Nodes.Next_Node (J);
      end loop;

      return Rule_Is_Terminal;
   end Is_Terminal;

   -----------------------
   -- Is_Lexer_Terminal --
   -----------------------

   function Is_Lexer_Terminal (N : Types.Node_Id) return Boolean is
      pragma Assert (EBNF.Nodes.Kind (N) = EBNF.Nodes.K_Syntax_Rule);

      I, J : Types.Node_Id;
      The_Syntactic_Primary : Types.Node_Id;
      The_Definitions_List : constant Types.Node_Id
        := EBNF.Nodes.Definitions_List (N);
      The_Rule_Name : constant Types.Node_Id
        := EBNF.Nodes.Meta_Identifier (N);

      Rule_Is_Lexer_Terminal : Boolean := True;

   begin
      Namet.Set_Name_Table_Byte
        (Normalize_Meta_Identifier (The_Rule_Name), 1);

      J := EBNF.Nodes.First_Node
        (EBNF.Nodes.Single_Definition (The_Definitions_List));

      while J /= Types.No_Node loop
         I := EBNF.Nodes.First_Node (EBNF.Nodes.Syntactic_Term (J));

         while I /= Types.No_Node loop
            The_Syntactic_Primary :=
              EBNF.Nodes.Syntactic_Primary (EBNF.Nodes.Syntactic_Factor (I));

            if EBNF.Nodes.Kind
              (EBNF.Nodes.A_Element
                 (The_Syntactic_Primary)) = EBNF.Nodes.K_Meta_Identifier
            then
               if Normalize_Meta_Identifier (The_Rule_Name)
                 = Normalize_Meta_Identifier
                 (EBNF.Nodes.A_Element (The_Syntactic_Primary))
               then
                  Rule_Is_Lexer_Terminal := False;
               end if;

               if Normalize_Meta_Identifier (The_Rule_Name)
                 /= Normalize_Meta_Identifier
                 (EBNF.Nodes.A_Element (The_Syntactic_Primary))
               then
                  if Namet.Get_Name_Table_Byte
                    (Normalize_Meta_Identifier
                       (EBNF.Nodes.A_Element (The_Syntactic_Primary))) = 1
                    or else not Is_Lexer_Terminal
                    (Namet.Get_Name_Table_Info
                       (Normalize_Meta_Identifier
                          (EBNF.Nodes.A_Element (The_Syntactic_Primary))))
                  then
                     Rule_Is_Lexer_Terminal := False;
                  end if;
               end if;
            elsif EBNF.Nodes.Kind
              (EBNF.Nodes.A_Element
                 (The_Syntactic_Primary)) = EBNF.Nodes.K_Empty_Sequence
              or else EBNF.Nodes.Kind
              (EBNF.Nodes.A_Element
                 (The_Syntactic_Primary)) = EBNF.Nodes.K_Optional_Sequence
              or else EBNF.Nodes.Kind
              (EBNF.Nodes.A_Element
                 (The_Syntactic_Primary)) = EBNF.Nodes.K_Repeated_Sequence
            then
               Rule_Is_Lexer_Terminal := False;
            end if;

            I := EBNF.Nodes.Next_Node (I);
         end loop;
         J := EBNF.Nodes.Next_Node (J);
      end loop;

      Namet.Set_Name_Table_Byte
        (Normalize_Meta_Identifier (The_Rule_Name), 2);

      return Rule_Is_Lexer_Terminal;
   end Is_Lexer_Terminal;

   -------------------------------
   -- Normalize_Meta_Identifier --
   -------------------------------

   function Normalize_Meta_Identifier (N : Types.Node_Id)
                                      return Types.Name_Id is
      The_Identifier : Types.List_Id;
      I : Types.Node_Id;
      Result : Types.Name_Id;
   begin
      The_Identifier := EBNF.Nodes.Identifier (N);
      I              := EBNF.Nodes.First_Node (The_Identifier);
      Namet.Name_Len := 0;
      loop
         Namet.Add_SparkStr_To_Name_Buffer
           (Namet.Name_String (EBNF.Nodes.Name (I)));
         I := EBNF.Nodes.Next_Node (I);
         exit when I = Types.No_Node;
         Namet.Add_Str_To_Name_Buffer ("_");
      end loop;
      Namet.Name_Find (Result);

      return Result;
   end Normalize_Meta_Identifier;

   -------------------
   -- Decorate_Tree --
   -------------------

   procedure Decorate_Tree (N : Types.Node_Id) is
      The_Syntax_Rules : constant Types.List_Id := EBNF.Nodes.Syntax_Rules (N);
      I : Types.Node_Id;
      The_Meta_Identifier : Types.Node_Id;
   begin
      --  1/ we check that there is only one definition for each
      --  production rule

      I := EBNF.Nodes.First_Node (The_Syntax_Rules);
      while I /= Types.No_Node loop
         The_Meta_Identifier := EBNF.Nodes.Meta_Identifier (I);
         if Namet.Get_Name_Table_Info
           (Normalize_Meta_Identifier (The_Meta_Identifier)) = 0 then
            Namet.Set_Name_Table_Info
              (Normalize_Meta_Identifier (The_Meta_Identifier), I);
         else
            Errors.Set_Error_Loc (1, EBNF.Nodes.Loc (I));
            Errors.Set_Error_Name
              (1, Normalize_Meta_Identifier (The_Meta_Identifier));
            Errors.Display_Error ("Duplicate production rule for #");
         end if;
         I := EBNF.Nodes.Next_Node (I);
      end loop;
   end Decorate_Tree;

   -----------------------
   -- Generate_Prologue --
   -----------------------

   procedure Generate_Prologue is
   begin
      Output.Write_Line ("////////////////////////////////////////////////");
      Output.Write_Line ("// This file has been generated automatically //");
      Output.Write_Str ("//");
      Output.Write_SparkStr (Spark.Command_Line.Command_Name);
      Output.Write_Eol;
      Output.Write_Line ("// Do not modify it !                         //");
      Output.Write_Line ("////////////////////////////////////////////////");
      Output.Write_Eol;

      Output.Write_Line ("module EBNF::Nodes {");

      Output.Increment_Indentation;
      Output.Write_Eol;
      Output.Write_Indentation;
      Output.Write_Line ("typedef long Name_Id;");
      Output.Write_Eol;

      Output.Write_Indentation;
      Output.Write_Line ("////////////////////");
      Output.Write_Indentation;
      Output.Write_Line ("// Internal nodes //");
      Output.Write_Indentation;
      Output.Write_Line ("////////////////////");
      Output.Write_Eol;

      Output.Write_Indentation;
      Output.Write_Line ("//  node_id:");
      Output.Write_Indentation;
      Output.Write_Line
        ("//    Node type from which all the other node types derive.");
      Output.Write_Indentation;
      Output.Write_Line ("//");
      Output.Write_Indentation;
      Output.Write_Line ("//  * next_node:");
      Output.Write_Indentation;
      Output.Write_Line
        ("//    Any node can be part of a container. Thus, the container");
      Output.Write_Indentation;
      Output.Write_Line
        ("//    includes a list of nodes. EBNF.Nodes.Next_Node"
           & " designates the next node");
      Output.Write_Indentation;
      Output.Write_Line
        ("//    in the list of the container. It is forbidden to define");
      Output.Write_Indentation;
      Output.Write_Line
        ("//    another attribute to point to a ""next"" node.");
      Output.Write_Eol;

      Output.Write_Indentation;
      Output.Write_Line ("interface node_id {");
      Output.Write_Indentation;
      Output.Write_Indentation;
      Output.Write_Line ("node_id  next_node;");
      Output.Write_Indentation;
      Output.Write_Line ("};");
      Output.Write_Eol;

      Output.Write_Indentation;
      Output.Write_Line ("//  list_id:");
      Output.Write_Indentation;
      Output.Write_Line
        ("//    List type used to represent the entities contained in a");
      Output.Write_Indentation;
      Output.Write_Line ("//    container.");
      Output.Write_Indentation;
      Output.Write_Line ("//");
      Output.Write_Indentation;
      Output.Write_Line ("//  * first_node, last_node:");
      Output.Write_Indentation;
      Output.Write_Line
        ("//    EBNF.Nodes.First and last nodes of a unidirectional list.");

      Output.Write_Eol;
      Output.Write_Indentation;
      Output.Write_Line ("interface list_id {");
      Output.Write_Indentation;
      Output.Write_Indentation;
      Output.Write_Line ("node_id  first_node;");
      Output.Write_Indentation;
      Output.Write_Indentation;
      Output.Write_Line ("node_id  last_node;");
      Output.Write_Indentation;
      Output.Write_Line ("};");
      Output.Write_Eol;
   end Generate_Prologue;

   --------------
   -- Generate --
   --------------

   procedure Generate (N : Types.Node_Id) is
      The_Syntax_Rules : constant Types.List_Id := EBNF.Nodes.Syntax_Rules (N);

      I : Types.Node_Id;
      The_Meta_Identifier : Types.Node_Id;
   begin
      Decorate_Tree (N);
      Generate_Prologue;

      Output.Write_Indentation;
      Output.Write_Line ("///////////////////////////////");
      Output.Write_Indentation;
      Output.Write_Line ("// Nodes specific to EBNF");
      Output.Write_Indentation;
      Output.Write_Line ("///////////////////////////////");
      Output.Write_Eol;

      I := EBNF.Nodes.First_Node (The_Syntax_Rules);
      while I /= Types.No_Node loop
         The_Meta_Identifier := EBNF.Nodes.Meta_Identifier (I);

         --  1/ Write the EBNF definition of the rule

         Output.Write_Indentation;
         Output.Write_Str ("// ");
         EBNF.Printer.Write (I);
         Output.Write_Eol;

         if not Is_Lexer_Terminal (I) then
            --         if not Is_Terminal (I) then

            --  2/ Header of the corresponding node

            Output.Write_Indentation;
            Output.Write_Str ("interface ");
            Namet.Write_Name (Normalize_Meta_Identifier (The_Meta_Identifier));
            Output.Write_Line (" : node_id {");

            --  3/

            declare
               The_Definitions_List : Types.Node_Id;
               K, J : Types.Node_Id;
               The_Syntactic_Primary : Types.Node_Id;
               The_Element : Types.Node_Id;
               Element_Name : Types.Name_Id;
            begin
               The_Definitions_List := EBNF.Nodes.Definitions_List (I);

               J := EBNF.Nodes.First_Node
                 (EBNF.Nodes.Single_Definition (The_Definitions_List));
               while J /= Types.No_Node loop
                  K := EBNF.Nodes.First_Node (EBNF.Nodes.Syntactic_Term (J));
                  while K /= Types.No_Node loop
                     The_Syntactic_Primary :=
                       EBNF.Nodes.Syntactic_Primary
                       (EBNF.Nodes.Syntactic_Factor (K));
                     The_Element := EBNF.Nodes.A_Element
                       (The_Syntactic_Primary);

                     if EBNF.Nodes.Kind (The_Element)
                       = EBNF.Nodes.K_Meta_Identifier
                     then
                        Element_Name := Normalize_Meta_Identifier
                          (The_Element);

                        if Is_Lexer_Terminal
                          (Namet.Get_Name_Table_Info (Element_Name))
                        then
                           if not Is_Terminal
                             (Namet.Get_Name_Table_Info (Element_Name))
                           then
                              Output.Write_Indentation;
                              Output.Write_Indentation;
                              Output.Write_Str ("name_id ");
                              Namet.Write_Name (Element_Name);
                              Output.Write_Str (";");
                              Output.Write_Eol;
                           end if;
                        else
                           Output.Write_Indentation;
                           Output.Write_Indentation;
                           Output.Write_Str ("node_id ");
                           Namet.Write_Name (Element_Name);
                           Output.Write_Str (";");
                           Output.Write_Eol;
                        end if;

                     elsif EBNF.Nodes.Kind (The_Element)
                       = EBNF.Nodes.K_Repeated_Sequence
                     then
                        declare
                           L, M : Types.Node_Id;
                        begin
                           L := EBNF.Nodes.First_Node
                             (EBNF.Nodes.Single_Definition
                                (EBNF.Nodes.Definitions_List (The_Element)));

                           while L /= Types.No_Node loop
                              M := EBNF.Nodes.First_Node
                                (EBNF.Nodes.Syntactic_Term (L));
                              while M /= Types.No_Node loop
                                 if not Is_Terminal
                                   (Namet.Get_Name_Table_Info
                                      (Normalize_Meta_Identifier
                                         (EBNF.Nodes.A_Element
                                            (EBNF.Nodes.Syntactic_Primary
                                               (EBNF.Nodes.Syntactic_Factor
                                                  (M))))))
                                   or else EBNF.Nodes.Next_Node (M)
                                   = Types.No_Node
                                 then
                                    Output.Write_Indentation;
                                    Output.Write_Indentation;
                                    if Is_Lexer_Terminal
                                      (Namet.Get_Name_Table_Info
                                         (Normalize_Meta_Identifier
                                            (EBNF.Nodes.A_Element
                                               (EBNF.Nodes.Syntactic_Primary
                                                  (EBNF.Nodes.Syntactic_Factor
                                                     (M))))))
                                    then
                                       Output.Write_Str ("name_id ");
                                    else
                                       Output.Write_Str ("list_id ");
                                    end if;

                                    Namet.Write_Name
                                      (Normalize_Meta_Identifier
                                         (EBNF.Nodes.A_Element
                                            (EBNF.Nodes.Syntactic_Primary
                                               (EBNF.Nodes.Syntactic_Factor
                                                  (M)))));

                                    Output.Write_Str (";");
                                    Output.Write_Eol;
                                 end if;

                                 M := EBNF.Nodes.Next_Node (M);
                              end loop;
                              L := EBNF.Nodes.Next_Node (L);
                           end loop;
                        end;

                     elsif EBNF.Nodes.Kind (The_Element)
                       = EBNF.Nodes.K_Optional_Sequence
                     then
                        declare
                           L, M : Types.Node_Id;
                        begin
                           L := EBNF.Nodes.First_Node
                             (EBNF.Nodes.Single_Definition
                                (EBNF.Nodes.Definitions_List (The_Element)));

                           while L /= Types.No_Node loop
                              M := EBNF.Nodes.First_Node
                                (EBNF.Nodes.Syntactic_Term (L));
                              while M /= Types.No_Node loop
                                 if not Is_Terminal
                                   (Namet.Get_Name_Table_Info
                                      (Normalize_Meta_Identifier
                                         (EBNF.Nodes.A_Element
                                            (EBNF.Nodes.Syntactic_Primary
                                               (EBNF.Nodes.Syntactic_Factor
                                                  (M))))))
                                 then
                                    Output.Write_Indentation;
                                    Output.Write_Indentation;
                                    Output.Write_Str ("node_id ");

                                    Namet.Write_Name
                                      (Normalize_Meta_Identifier
                                         (EBNF.Nodes.A_Element
                                            (EBNF.Nodes.Syntactic_Primary
                                               (EBNF.Nodes.Syntactic_Factor
                                                  (M)))));

                                    Output.Write_Str (";");
                                    Output.Write_Eol;
                                 end if;

                                 M := EBNF.Nodes.Next_Node (M);
                              end loop;
                              L := EBNF.Nodes.Next_Node (L);
                           end loop;
                        end;
                     end if;
                     K := EBNF.Nodes.Next_Node (K);
                  end loop;
                  J := EBNF.Nodes.Next_Node (J);
               end loop;
            end;

            --  4/

            Output.Write_Indentation;
            Output.Write_Str ("};");
            Output.Write_Eol;
            Output.Write_Eol;
         end if;

         I := EBNF.Nodes.Next_Node (I);
      end loop;
      Output.Decrement_Indentation;
      Output.Write_Line ("};");
   end Generate;

end EBNF2IDL;
