------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                               E R R O R S                                --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Locations;
with Types;
--# inherit Locations,
--#         Namet,
--#         Output,
--#         Spark.Command_Line,
--#         Spark.OS_Lib,
--#         Spark.Strings,
--#         Types;

package Errors
   --# own Error_Int,
   --#     Error_Loc,
   --#     Error_Name,
   --#     N_Errors,
   --#     N_Warnings;
   --# initializes Error_Int,
   --#             Error_loc,
   --#             Error_Name,
   --#             N_Errors,
   --#             N_Warnings;
is

   procedure Display_Status;
   --# global in     N_Warnings;
   --#        in out Error_Int;
   --#        in out Error_Loc;
   --#        in out Error_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Errors;
   --#        in out Output.State;
   --  Display the number of errors, warnings, and set error code accordingly.
   --  Note: it should be called only once, and be the last function called.

   procedure Display_Error (S : String);
   --# global in out Error_Int;
   --#        in out Error_Loc;
   --#        in out Error_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Errors;
   --#        in out Output.State;

   procedure Display_Warning (S : String);
   --# global in out Error_Int;
   --#        in out Error_Loc;
   --#        in out Error_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out N_Warnings;
   --#        in out Output.State;

   --  Display an error and output error message S. S may include
   --  meta-characters.
   --
   --  '%' designates a string representing Error_Name (N) where N is
   --  the number of '%' and '#' in the substring.
   --
   --  '#' designates a quoted string representing Error_Name (N).
   --
   --  '!' designates a location representing Error_Loc (L) where L is
   --  the number of '!' in the substring.
   --
   --  '$' designates an integer representing Error_Int (I) where I is
   --  the number of '$' in the substring.

   procedure Initialize;
   --# global out Error_Int;
   --#        out Error_Loc;
   --#        out Error_Name;

   Error_Max : constant := 2;
   subtype Error_Range is Integer range 1 .. Error_Max;

   type Error_Locs is array (Error_Range) of Locations.Location;
   type Error_Ints is array (Error_Range) of Types.Nat;
   type Error_Names is array (Error_Range) of Types.Name_Id;

   Error_Loc  : Error_Locs  :=
      Error_Locs'(Error_Range => Locations.No_Location);
   Error_Int  : Error_Ints  := Error_Ints'(Error_Range => 0);
   Error_Name : Error_Names := Error_Names'(Error_Range => Types.No_Name);

   N_Errors   : Types.Nat := 0;
   N_Warnings : Types.Nat := 0;

   procedure Set_Error_Loc
     (Position      : Error_Range;
      The_Error_Loc : Locations.Location);
   --# global in out Error_Loc;

   procedure Set_Error_Name
     (Position       : Error_Range;
      The_Error_Name : Types.Name_Id);
   --# global in out Error_Name;

private

   type Message_Type is (Notice, Warning, Error);

   procedure Internal_Display_Message (Msg_Type : Message_Type; S : String);
   --# global in out Error_Int;
   --#        in out Error_Loc;
   --#        in out Error_Name;
   --#        in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;
   --#        in out Namet.State;
   --#        in out Output.State;

   procedure Check_Space;
   --# global in out Namet.Name_Buffer;
   --#        in out Namet.Name_Len;

   --  Check there is a trailing space in order to append a string to
   --  name buffer.

end Errors;
