------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                            L O C A T I O N S                             --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Namet;
with Types; use type Types.Name_Id, Types.Int;

package body Locations is

   --------------------
   -- Location_Image --
   --------------------

   procedure Location_Image
     (Loc : Location;
      Img : out Spark.Strings.SparkString)
   is
      Column : Types.Nat;
      Backup : Types.Name_Id;
      --  Backup name buffer
      Result : Types.Name_Id;
      --  Store returned value before restoring name buffer

      Result_String : Spark.Strings.SparkString;
   begin
      if Loc.Last_Pos > Loc.First_Pos then
         Column :=
           Types.Nat'((Types.Nat (Loc.Last_Pos) - Types.Nat (Loc.First_Pos)) +
                      1);
      else
         Column := 1;
      end if;

      if Loc.Base_Name = Types.No_Name then
         Result_String := Spark.Strings.EmptyString;

      else
         --  A critical issue consist in preserving Name_Buffer. In
         --  other words, this function must not have side effect.

         --  Save name buffer to restore it later on
         if Namet.Name_Len > 0 then
            Namet.Name_Find (Backup);
         else
            Backup := Types.No_Name;
         end if;

         Namet.Get_Name_String (Loc.Base_Name);
         Namet.Add_Char_To_Name_Buffer (':');
         Namet.Add_Nat_To_Name_Buffer (Loc.Line);
         Namet.Add_Char_To_Name_Buffer (':');
         if Column < 10 then
            Namet.Add_Char_To_Name_Buffer ('0');
         end if;
         Namet.Add_Nat_To_Name_Buffer (Column);
         Namet.Name_Find (Result);

         --  Restore backup into name buffer
         if Backup /= Types.No_Name then
            Namet.Get_Name_String (Backup);
         end if;

         --  Return result using a Get_Name_String variant with no side effect
         Result_String := Namet.Name_String (Result);
      end if;

      Img := Result_String;
   end Location_Image;

end Locations;
