------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                               O U T P U T                                --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Spark.Strings;
with System;
with Types;         use type Types.Int;
--# inherit Ada.Characters.Latin_1,
--#         Spark.OS_Lib,
--#         Spark.Strings,
--#         Spark.Type_Image,
--#         System,
--#         Types;

package Output
   --# own N_Space,
   --#     Space_Increment,
   --#     State;
   --# initializes N_Space,
   --#             Space_Increment,
   --#             State;
is

   procedure Set_Output (New_Output : System.Address);
   --# global in out State;
   --  Sets subsequent output to appear on the given file

   procedure Set_Standard_Error;
   --# global in out State;
   --  Sets subsequent output to appear on the standard error file
   --  (whatever that might mean for the host operating system, if
   --  anything).

   procedure Set_Standard_Output;
   --# global in out State;
   --  Sets subsequent output to appear on the standard output file
   --  (whatever that might mean for the host operating system, if
   --  anything). Output to standard output is the default mode before
   --  any call to either of the Set procedures.

   function Current_Output return System.Address;
   --# global in State;
   --  Return the current output

   procedure Write_Char (C : Character);
   --# global in out State;
   --  Write one character to the standard output file. Note that the
   --  character should not be LF or CR (use Write_Eol for end of line)

   procedure Write_Eol;
   --# global in out State;
   --  Write an end of line (whatever is required by the system in use,
   --  e.g. CR/LF for DOS, or LF for Unix) to the standard output file.
   --  This routine also empties the line buffer, actually writing it
   --  to the file. Note that Write_Eol is the only routine that causes
   --  any actual output to be written.

   procedure Write_Int (Val : Types.Int);
   --# global in out State;
   --  Write an integer value with no leading blanks or zeroes. Negative
   --  values are preceded by a minus sign).

   procedure Write_Str (S : String);
   --# global in out State;

   procedure Write_SparkStr (S : Spark.Strings.SparkString);
   --# global in out State;
   --  Write a string of characters to the standard output file. Note that
   --  end of line is handled separately using WRITE_EOL, so the string
   --  should not contain either of the characters LF or CR, but it may
   --  contain horizontal tab characters.

   procedure Write_Line (S : String);
   --# global in out State;
   --  Equivalent to Write_Str (S) followed by Write_Eol;

   subtype Increment_Type is Natural range 0 .. 10;

   Space_Increment : Increment_Type := 2;
   N_Space         : Natural        := 0;

   procedure Decrement_Indentation;
   --# global in     Space_Increment;
   --#        in out N_Space;

   procedure Increment_Indentation;
   --# global in     Space_Increment;
   --#        in out N_Space;

   procedure Set_Space_Increment (Value : Increment_Type);
   --# global out Space_Increment;

   procedure Write_Indentation;
   --# global in     N_Space;
   --#        in out State;

private

   procedure Flush_Buffer;
   --# global in out State;
   --  Flush buffer if non-empty and reset column counter

end Output;
