------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                         S P A R K . O S _ L I B                          --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Ada.Characters.Latin_1;

package body Spark.OS_Lib is

   ---------------
   -- Open_Read --
   ---------------

   function FOpen_Read (Name  : Strings.SparkString) return System.Address is
      --  Note: we use a hide annotation since slicing is forbidden by
      --  Spark.
      --# hide FOpen_Read

      C_Name : String (1 .. Name.Length + 1);
   begin
      C_Name (1 .. Name.Length) := Name.Text (1 .. Name.Length);
      C_Name (C_Name'Last)      := Ada.Characters.Latin_1.NUL;
      return fopen (C_Name, "r" & Ada.Characters.Latin_1.NUL);
   end FOpen_Read;

   ------------
   -- stdout --
   ------------

   function stdout return System.Address is
      --  Note we use a hide annotation since C_Stdout is not fully
      --  defined in the Ada body.

      --  XXX We could certainly defined it as a global variable in
      --  the spec, but this would adds lot of new annotations, to be
      --  assessed .. same for being in the body ..

      --# hide stdout
      C_Stdout : System.Address;
      pragma Import (C, C_Stdout,  "stdout");
   begin
      return C_Stdout;
   end stdout;

   ------------
   -- stderr --
   ------------

   function stderr return System.Address is
      --  Note we use a hide annotation since C_Stdout is not fully
      --  defined in the Ada body.
      --# hide stderr

      C_Stderr : System.Address;
      pragma Import (C, C_Stderr,  "stderr");
   begin
      return C_Stderr;
   end stderr;

end Spark.OS_Lib;
