------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                         S P A R K . O S _ L I B                          --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

--  This package is a thin binding of C functions defined in the C
--  Reference Manual ISO-IEC ISO/IEC 9899:TC2 standard.

with System;
with Spark.Strings;
--# inherit Ada.Characters.Latin_1,
--#         Spark.Strings,
--#         System;

package Spark.OS_Lib is

   type size_t is mod 2 ** System.Word_Size;

   ---------------
   -- <stdio.h> --
   ---------------

   --  XXX Spark does not allow constructions like
   --      type is new System.Address, and
   --  subtype of private types are not allowed either

   function FOpen_Read (Name : Strings.SparkString) return System.Address;

   function stdout return System.Address;
   function stderr return System.Address;
   --  See 7.19.1 of ISO/IEC 9899:TC2

   function fclose (Stream : System.Address) return Integer;
   pragma Import (C, fclose);
   --  See 7.19.5.1 of ISO/IEC 9899:TC2

   function fopen (Path : String; FMode : String) return System.Address;
   pragma Import (C, fopen);
   --  See 7.19.5.3 of ISO/IEC 9899:TC2

   function fgetc (stream : System.Address) return Integer;
   --# return Result => Result >= 0 and Result <= 255;
   pragma Import (C, fgetc);
   --  See 7.19.7.1 of ISO/IEC 9899:TC2

   procedure fputc (C : Integer; Stream : System.Address);
   pragma Import (C, fputc);
   --  See 7.19.7.3 of ISO/IEC 9899:TC2

   function feof (stream : System.Address) return Integer;
   pragma Import (C, feof);
   --  See 7.19.10.2 of ISO/IEC 9899:TC2

   ----------------
   -- <stdlib.h> --
   ----------------

   procedure OS_Exit (Status : Integer);
   pragma Import (C, OS_Exit, "exit");
   pragma No_Return (OS_Exit);
   --  See 7.20.4.3 of ISO/IEC 9899:TC2

end Spark.OS_Lib;
