------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                        S P A R K . S T R I N G S                         --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

package body Spark.Strings is

   ----------------
   -- CopyString --
   ----------------

   procedure CopyString (DestStr : out SparkString; SrcStr : String) is
      ResultLength : Lengths;
      J            : Positive;
   begin
      DestStr := EmptyString;
      if SrcStr'Length < MaxStringLength then
         ResultLength := SrcStr'Length;
      else
         ResultLength := MaxStringLength;
      end if;
      --# assert ResultLength <= SrcStr'Length;

      if ResultLength = 0 then
         DestStr := EmptyString;
      else
         J := Positions'First;
         loop
         --# assert J <= ResultLength + 1
         --#    and ResultLength <= SrcStr'Length;
            exit when J > ResultLength;

            --# assert J <= Positions'last
            --#    and J <= ResultLength
            --#    and ResultLength <= SrcStr'Length;
            DestStr.Text (J) := SrcStr (J);
            J                := J + 1;
         end loop;

         DestStr.Length := ResultLength;
      end if;
   end CopyString;

   ---------------
   -- AppenChar --
   ---------------

   procedure AppendChar
     (Str     : in out SparkString;
      Ch      : Character;
      Success : out Boolean)
   is
      ResultString : StringText;
      ResultLength : Lengths;
   begin
      ResultString := Str.Text;
      ResultLength := Str.Length;

      if ResultLength = MaxStringLength then
         Success := False;
      else
         Success                     := True;
         ResultLength                := ResultLength + 1;
         ResultString (ResultLength) := Ch;
      end if;

      Str.Text   := ResultString;
      Str.Length := ResultLength;
   end AppendChar;

   ---------------
   -- Substring --
   ---------------

   function Substring
     (Str  : SparkString;
      Pos1 : Positions;
      Pos2 : Positions)
      return SparkString
   is
      NewString : SparkString;
      Buffer    : StringText;
      BLen      : Positions;
      J         : Positions;
   begin
      if Pos2 - Pos1 < 0 then
         NewString := EmptyString;

      else
         --# assert Pos2 - Pos1 >= 0;
         BLen := (Pos2 - Pos1) + Positions'First;

         if BLen = 255 then
            NewString := Str;

         else
            --# assert BLen <= 254
            --#    and BLen > 0
            --#    and BLen = Pos2 - Pos1 + 1;
            Buffer := StringText'(others => ' ');

            J := Positions'First;
            loop
            --# assert BLen <= 254
            --#    and J <= BLen + 1
            --#    and BLen = Pos2 - Pos1 + 1;
               exit when J > BLen;

               --# assert J <= BLen
               --#    and Pos1 + J - 1 in Positions
               --#    and BLen <= 254
               --#    and BLen = Pos2 - Pos1 + 1;

               Buffer (J) := Str.Text (Pos1 + (J - 1));
               J          := J + 1;
            end loop;

            NewString := SparkString'(Length => BLen, Text => Buffer);
         end if;
      end if;

      return NewString;
   end Substring;

end Spark.Strings;
