------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                        S P A R K . S T R I N G S                         --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

package Spark.Strings is

   subtype Lengths is Integer range 0 .. 255;
   MaxStringLength : constant Lengths := Lengths'Last;
   subtype Positions is Lengths range 1 .. MaxStringLength;
   subtype StringText is String (Positions);

   type SparkString is record
      Length : Lengths;
      Text   : StringText;
   end record;

   EmptyString : constant SparkString :=
      SparkString'(Length => 0, Text => StringText'(Positions => ' '));

   procedure AppendChar
     (Str     : in out SparkString;
      Ch      : Character;
      Success : out Boolean);

   function Substring
     (Str  : SparkString;
      Pos1 : Positions;
      Pos2 : Positions)
      return SparkString;
   --  Return the substring of Str starting at Pos1 and ending at (and
   --  including) Pos2

   procedure CopyString (DestStr : out SparkString; SrcStr : String);

end Spark.Strings;
