------------------------------------------------------------------------------
--                                                                          --
--                         BlackShield COMPONENTS                           --
--                                                                          --
--                G E N E R A T E _ S P A R K _ C O N F I G                 --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 2010, J. Hugues.                       --
--                                                                          --
-- BlackShield is free software;  you  can  redistribute  it and/or  modify --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. BlackShield is distributed  in  the  hope  that it will be--
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with BlackShield; see file COPYING--
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--   BlackShield is maintained by Jerome Hugues <hugues.jerome@gmail.com>   --
--                                                                          --
------------------------------------------------------------------------------

with Ada.Text_IO; use Ada.Text_IO;
with System;

procedure Generate_SPARK_Config is

   type T is range System.Min_Int .. System.Max_Int;
   type Tmod is mod System.Max_Binary_Modulus;
   type Tfloat is digits System.Max_Base_Digits range -1.0 .. 1.0;

begin

   ----------------------
   -- Package Standard --
   ----------------------

   --  Note, for now, we list only the types required to perform some
   --  proof, for more information on required types, see SPARK
   --  Examiner User Guide.

   Put_Line ("--  This package has been automatically generated by "
               & "Generate_SPARK_Config");
   Put_Line ("--  DO NOT EDIT");
   New_Line;

   Put_Line ("package Standard is");

   Put_Line ("   type Integer is range " &
               Integer'Image (Integer'First) & " .." &
               Integer'Image (Integer'Last) & ";");

   Put_Line ("   type Short_Short_Integer is range " &
               Short_Short_Integer'Image (Short_Short_Integer'First) &
               " .. " &
               Short_Short_Integer'Image (Short_Short_Integer'Last) & ";");
   Put_Line ("   type Short_Integer is range " &
               Short_Integer'Image (Short_Integer'First) & " .. " &
               Short_Integer'Image (Short_Integer'Last) & ";");
   Put_Line ("   type Long_Integer is range " &
               Long_Integer'Image (Long_Integer'First) & " .. " &
               Long_Integer'Image (Long_Integer'Last) & ";");
   Put_Line ("   type Long_Long_Integer is range " &
               Long_Long_Integer'Image (Long_Long_Integer'First) & " .. " &
               Long_Long_Integer'Image (Long_Long_Integer'Last) & ";");
   Put_Line ("end Standard;");
   New_Line;

   --------------------
   -- Package System --
   --------------------

   Put_Line ("package System is");

   Put_Line ("   type Address is private;");
   Put_Line ("   Min_Int : constant := " & T'Image (System.Min_Int) & ";");
   Put_Line ("   Max_Int : constant := " & T'Image (System.Max_Int) & ";");

   Put_Line ("   Max_Binary_Modulus : constant := " &
             Tmod'Image (Tmod'Last) & " + 1;");
   Put_Line ("   Max_Digits : constant := " &
             T'Image (System.Max_Digits) & ";");
   Put_Line ("   Max_Base_Digits : constant := " &
             T'Image (System.Max_Base_Digits) & ";");
   Put_Line ("   Max_Mantissa : constant := " &
             T'Image (System.Max_Mantissa) & ";");
   Put_Line ("   Storage_Unit : constant := " &
             T'Image (System.Storage_Unit) & ";");
   Put_Line ("   Word_Size : constant := " &
             T'Image (System.Word_Size) & ";");
   Put_Line ("   Fine_Delta : constant := " &
             Tfloat'Image (System.Fine_Delta) & ";");
   Put_Line ("   subtype Any_Priority is Integer range " &
             Integer'Image (System.Any_Priority'First) & " .. " &
             Integer'Image (System.Any_Priority'Last) & ";");
   Put_Line ("   subtype Priority is Any_Priority range " &
             Integer'Image (System.Priority'First) & " .. " &
             Integer'Image (System.Priority'Last) & ";");
   Put_Line ("   subtype Interrupt_Priority is Any_Priority range " &
             Integer'Image (System.Interrupt_Priority'First) & " .. " &
             Integer'Image (System.Interrupt_Priority'Last) & ";");
   Put_Line ("   Default_Bit_Order : constant Bit_Order := " &
             System.Bit_Order'Image (System.Default_Bit_Order) & ";");
   Put_Line ("end System;");

end Generate_SPARK_Config;
